@@ -5,85 +5,76 @@ upload, refresh, or making backups.
 This module provides that ability to access task specific attributes
 and optionally poll for status of an operation.
 
-An example of using a task poller (use 'wait_for_finish=True')::
+An example of using a task poller when uploading an engine policy
+(use `wait_for_finish=True`)::
 
     engine = Engine('myfirewall')
     poller = engine.upload(policy=fwpolicy, wait_for_finish=True)
     while not poller.done():
         poller.wait(5)
         print("Task Progress {}%".format(poller.task.progress))
-    print(poller.message())
+    print(poller.last_message())
 
 """
 import re
 import time
-import datetime
 import threading
-from smc import session
-from smc.base.model import prepared_request, SimpleElement, Element
+from smc.base.model import ElementCache, Element, SubElement
 from smc.api.exceptions import TaskRunFailed, ActionCommandFailed,\
     ResourceNotFound
+from smc.base.collection import Search
+from smc.base.util import millis_to_utc
+
 
 clean_html = re.compile(r'<.*?>')
 
 
-def millis_to_utc(millis):
-    return datetime.datetime(1970, 1, 1) + datetime.timedelta(milliseconds=millis)
-
-
-def task_history():
+def TaskHistory():
     """
-    Get all tasks stored by the SMC
-
-    Example of aborting an existing task by follower link::
-
-        for task in task_history():
-            if task.follower == task:
-                task.abort()
-
-    Abort any in progress tasks::
-
-        for task in task_history():
-            if task.in_progress:
-                task.abort()
-
-    :return: list :py:class:`~Task`
+    Task history retrieves a list of tasks in an event queue.
+    
+    :return: list of task events
+    :rtype: TaskProgress
     """
-    tasks = prepared_request(
-        href=session.entry_points.get('task_progress')
-        ).read().json
+    events = Search.objects.entry_point('task_progress')
+    return [event.task for event in events]
+        
 
-    return [Task(prepared_request(href=task['href']).read().json)
-            for task in tasks]
-
-
-def task_status(follower):
+class TaskProgress(Element):
     """
-    Return the task specified.
-
-    Return specific task::
-
-        task = task_status('http://......')
-
-    :param str follower: task follower link
-    :return: :py:class:`~Task`
+    Task Progress represents a task event queue. These
+    tasks may be completed or still running. The task event
+    queue events can be retrieved by calling :func:`~TaskHistory`.
     """
-    for task in task_history():
-        if task.href == follower:
-            return task
+    typeof = 'task_progress'
 
+    @property
+    def task(self):
+        """
+        Return the task associated with this event
+        
+        :rtype: Task
+        """
+        return Task(self.data)
+        
 
-class Task(object):
+class Task(SubElement):
     """
     Task representation. This is generic and the format is used for
     any calls to SMC that return an asynchronous follower link to
-    check the status of the task. Use this as input to the task_handler
-    generator to control displaying updates to the user.
+    check the status of the task.
 
-    :param dict task: task json after task process started
+    :param str last_message: Last message received on this task
+    :param bool in_progress: Whether the task is in progress or finished
+    :param bool success: Whether the task succeeded or not
+    :param str follower: Fully qualified path to the follower link to track
+        this task.
     """
     def __init__(self, task):
-        self.data = SimpleElement(**task)
+        super(Task, self).__init__(
+            href=task.get('follower', None),
+            name=task.get('type', None))
+        self.data = ElementCache(task)
 
     @property
     def resource(self):
@@ -93,20 +84,7 @@ class Task(object):
         :rtype: list(Element)
         """
         return [Element.from_href(resource)
-                for resource in self.data['resource']]
-
-    @property
-    def href(self):
-        return self.data['follower']
-
-    @property
-    def last_message(self):
-        """
-        Return the last message logged by SMC
-
-        :rtype: str
-        """
-        return self.data['last_message']
+                for resource in self.data.get('resource', [])]
 
     @property
     def progress(self):
@@ -117,24 +95,6 @@ class Task(object):
         """
         return self.data.get('progress', 0)
 
-    @property
-    def in_progress(self):
-        """
-        Is this task in progress or complete
-
-        :rtype: bool
-        """
-        return self.data['in_progress']
-
-    @property
-    def success(self):
-        """
-        Was the task completion considered successful
-
-        :rtype: bool
-        """
-        return self.data['success']
-
     @property
     def start_time(self):
         """
@@ -165,10 +125,10 @@ class Task(object):
         :return: None
         """
         try:
-            prepared_request(
-                ActionCommandFailed,
-                href=self.data.get_link('abort')
-                ).delete()
+            self.make_request(
+                method='delete',
+                resource='abort')
+    
         except ResourceNotFound:
             pass
         except ActionCommandFailed:
@@ -181,35 +141,72 @@ class Task(object):
 
         :rtype: str
         """
-        return self.data.get_link('result')
+        return self.get_relation('result')
 
     def update_status(self):
         """
         Gets the current status of this task and returns a
         new task object.
 
-        :raises TaskRunFailed
+        :raises TaskRunFailed: fail to update task status
         """
-        task = prepared_request(
+        task = self.make_request(
             TaskRunFailed,
-            href=self.href).read().json
+            href=self.href)
+
         return Task(task)
-
+    
     def __getattr__(self, key):
-        try:
-            return self.data[key]
-        except KeyError:
-            pass
+        return self.data.get(key)
+    
+    @staticmethod
+    def execute(self, resource, **kw):
+        """
+        Execute the task and return a TaskOperationPoller.
+        
+        :rtype: TaskOperationPoller
+        """
+        params = kw.pop('params', {})
+        json = kw.pop('json', None)
+        task = self.make_request(
+            TaskRunFailed,
+            method='create',
+            params=params,
+            json=json,
+            resource=resource)
 
-    def __repr__(self):
-        return '{0}(type={1})'.format(
-            self.__class__.__name__, self.type)
+        timeout = kw.pop('timeout', 5)
+        wait_for_finish = kw.pop('wait_for_finish', True)
+        
+        return TaskOperationPoller(
+            task=task, timeout=timeout,
+            wait_for_finish=wait_for_finish,
+            **kw)
+
+    @staticmethod
+    def download(self, resource, filename, **kw):
+        """
+        Start and return a Download Task
+        
+        :rtype: DownloadTask(TaskOperationPoller)
+        """
+        params = kw.pop('params', {})
+        task = self.make_request(
+            TaskRunFailed,
+            method='create',
+            resource=resource,
+            params=params)
+
+        return DownloadTask(
+            filename=filename, task=task)
 
 
 class TaskOperationPoller(object):
     """
     Task Operation Poller provides a way to poll the SMC
-    for the status of the task operation.
+    for the status of the task operation. This is returned
+    by functions that return a task. Typically these will be
+    operations like refreshing policy, uploading policy, etc.
     """
     def __init__(self, task, timeout=5, max_tries=36,
                  wait_for_finish=False):
@@ -251,7 +248,8 @@ class TaskOperationPoller(object):
         The callable must take 1 argument which will be
         the completed Task.
 
-        :param callable callback
+        :param callback: a callable that takes a single argument which
+            will be the completed Task.
         """
         if self._done is None or self._done.is_set():
             raise ValueError('Task has already finished')
@@ -313,7 +311,7 @@ class TaskOperationPoller(object):
 
 class DownloadTask(TaskOperationPoller):
     """
-    A download task handles tasks that have files assocaited, for example
+    A download task handles tasks that have files associated, for example
     exporting an element to a specified file.
     """
     def __init__(self, filename, task, **kw):
@@ -325,12 +323,17 @@ class DownloadTask(TaskOperationPoller):
 
     def download(self, timeout):
         self.wait(timeout)
+        if not self.task.in_progress and not self.task.success:
+            raise TaskRunFailed(self.task.last_message)
         try:
-            location = prepared_request(
-                ActionCommandFailed,
+            result = self.task.make_request(
+                TaskRunFailed,
+                raw_result=True,
                 href=self.task.result_url,
-                filename=self.filename).read()
-            self.filename = location.content
+                filename=self.filename)
+
+            self.filename = result.content
+    
         except IOError as io:
             raise TaskRunFailed(
                 'Export task failed with message: {}'.format(io))
