@@ -1,45 +1,34 @@
 """
 Middle tier helper module to wrap CRUD operations and catch exceptions
 
-SMCRequest is the general data structure that is sent to the prepared_request
+SMCRequest is the general data structure that is sent to the send_request
 method in smc.api.web.SMCConnection to submit the data to the SMC.
 """
 import logging
-from smc import session
-from smc.api.exceptions import SMCOperationFailure, SMCConnectionError,\
-    UnsupportedEntryPoint
+from smc.api.exceptions import SMCOperationFailure, SMCConnectionError
 from smc.base.util import unicode_to_bytes
 
 logger = logging.getLogger(__name__)
 
 
-def method(method):
-    def _method(f):
-        def wrapper(self, *args):
-            setattr(self, '_method', method)
-            return f(self, *args)
-        return wrapper
-    return _method
+def _get_default_session():
+    from smc import session
+    return session
 
 
 class _RequestHandler(object):
     def __init__(self, **kwargs):
-        self._method = None
         self.files = None
-        self.headers = {'content-type': 'application/json'}
+        self.headers = {'Content-Type': 'application/json'}
 
-    @property
-    def method(self):
-        return self._method
-
-    def _make_request(self):
+    def _make_request(self, method):
         err = None
         result = None
         try:
-            if self.method == 'GET':
+            if method == 'GET':
                 if not self.href:
-                    self.href = session.entry_points.get('elements')
-            result = session.connection.send_request(self.method, self)
+                    self.href = _get_default_session().entry_points.get('elements')
+            result = _get_default_session().connection.send_request(method, self)
 
         except SMCOperationFailure as e:
             result = e.smcresult
@@ -49,14 +38,13 @@ class _RequestHandler(object):
                 pass
         except SMCConnectionError as e:
             err = e
-        except TypeError as e:
-            err = e
         except IOError as e:
             err = e
+        except TypeError as e:
+            err = e
         finally:
             if err:
                 raise err
-            logger.debug(result)
             return result
 
 
@@ -89,24 +77,26 @@ class SMCRequest(_RequestHandler):
         for k, v in kwargs.items():
             setattr(self, k, v)
 
-    @method('POST')
     def create(self):
-        return self._make_request()
+        return self._make_request(method='POST')
 
-    @method('DELETE')
     def delete(self):
-        return self._make_request()
+        return self._make_request(method='DELETE')
 
-    @method('PUT')
     def update(self):
-        return self._make_request()
+        return self._make_request(method='PUT')
 
-    @method('GET')
     def read(self):
-        return self._make_request()
+        return self._make_request(method='GET')
 
-    def __repr__(self):
-        return '<SMCRequest [%s]>' % (self.method)
+    def __str__(self):
+        sb = []
+        for key in self.__dict__:
+            sb.append(
+                "{key}='{value}'".format(
+                    key=key,
+                    value=self.__dict__[key]))
+        return 'SMCRequest({})'.format(','.join(sb))    
 
 
 def fetch_entry_point(name):
@@ -117,30 +107,14 @@ def fetch_entry_point(name):
 
     :method: GET
     :param str name: valid element entry point, i.e. 'host', 'iprange', etc
+    :raises UnsupportedEntryPoint: entry point not available in this API version
     :return: href pulled from API cache
     :rtype: str
     """
-    try:
-        return session.entry_points.get(name)  # from entry point cache
-    except UnsupportedEntryPoint:
-        raise
+    return _get_default_session().entry_points.get(name)  # from entry point cache
 
-
-def fetch_no_filter(entry_point, filter=None):  # @ReservedAssignment
-    """
-    Fetch elements with no filter_context filter. This will pull from the
-    entry point base.
-    """
-    return SMCRequest(
-        href=session.entry_points.get(entry_point),
-        params={'filter': filter}
-        ).read().json
     
-
-def fetch_href_by_name(name,
-                       filter_context=None,
-                       exact_match=True,
-                       domain=None):
+def fetch_href_by_name(name, filter_context=None, exact_match=True, domain=None):
     """
     Find the element based on name and optional filters. By default, the
     name provided uses the standard filter query. Additional filters can
@@ -201,8 +175,7 @@ def fetch_json_by_href(href, params=None):
     :params dict params: optional search query parameters
     :return: :py:class:`smc.api.web.SMCResult`
     """
-    result = SMCRequest(href=href,
-                        params=params).read()
+    result = SMCRequest(href=href, params=params).read()
     if result:
         result.href = href
     return result
@@ -218,3 +191,4 @@ def fetch_json_by_post(href, json=None):
     """
     return SMCRequest(href=href,
                       json=json).create()
+
