@@ -3,6 +3,7 @@ Configuration Loader
 """
 import os
 import io
+import json
 from smc.api.exceptions import ConfigLoadError
 
 try:
@@ -23,9 +24,15 @@ def load_from_environ():
         SMC_CLIENT_CERT=path/to/cert
         SMC_TIMEOUT = 30 (seconds)
         SMC_API_VERSION = 6.1 (optional - uses latest by default)
-        SMC_DOMAIN = name of domain, Shared is default 
+        SMC_DOMAIN = name of domain, Shared is default
+        SMC_EXTRA_ARGS = string in dict format of extra args needed
     
     SMC_CLIENT CERT is only checked IF the SMC_URL is an HTTPS url.
+    
+    Example of forcing retry on service unavailable::
+    
+        os.environ['SMC_EXTRA_ARGS'] = '{"retry_on_busy": "True"}'
+    
     """
     try:
         from urllib.parse import urlparse
@@ -37,14 +44,22 @@ def load_from_environ():
     smc_timeout = os.environ.get('SMC_TIMEOUT', None)
     api_version = os.environ.get('SMC_API_VERSION', None)
     domain = os.environ.get('SMC_DOMAIN', None)
+    smc_extra_args = os.environ.get('SMC_EXTRA_ARGS', '')
     
     if not smc_apikey or not smc_address:
         raise ConfigLoadError(
             'If loading from environment variables, you must provide values '
             'SMC_ADDRESS and SMC_API_KEY.')
-        
+    
     config_dict = {}
-        
+    
+    if smc_extra_args:
+        try:
+            args_as_dict = json.loads(smc_extra_args)
+            config_dict.update(args_as_dict)
+        except ValueError:
+            pass
+   
     config_dict.update(smc_apikey=smc_apikey)
     config_dict.update(timeout=smc_timeout)
     config_dict.update(api_version=api_version)
@@ -70,6 +85,7 @@ def load_from_environ():
     
     return transform_login(config_dict)
 
+
 def load_from_file(alt_filepath=None):
     """ Attempt to read the SMC configuration from a
     dot(.) file in the users home directory. The order in
@@ -77,6 +93,7 @@ def load_from_file(alt_filepath=None):
 
     - Passing credentials as parameters to the session login
     - Shared credential file (~/.smcrc)
+    - Environment variables
 
     :param alt_filepath: Specify a different file name for the
            configuration file. This should be fully qualified and include
@@ -91,6 +108,7 @@ def load_from_file(alt_filepath=None):
         smc_port=8082
         smc_ssl=True
         verify_ssl=True
+        retry_on_busy=True
         ssl_cert_file='/Users/davidlepage/home/mycacert.pem'
 
     :param str smc_address: IP of the SMC Server
@@ -99,6 +117,7 @@ def load_from_file(alt_filepath=None):
     :param int smc_port: port to use for SMC, (default: 8082)
     :param bool smc_ssl: Whether to use SSL (default: False)
     :param bool verify_ssl: Verify client cert (default: False)
+    :param bool retry_on_busy: Retry CRUD operation if service is unavailable (default: False)
     :param str ssl_cert_file: Full path to client pem (default: None)
 
     The only settings that are required are smc_address and smc_apikey.
@@ -111,12 +130,13 @@ def load_from_file(alt_filepath=None):
 
     """
     required = ['smc_address', 'smc_apikey']
-    bool_type = ['smc_ssl', 'verify_ssl']  # boolean option flag
+    bool_type = ['smc_ssl', 'verify_ssl', 'retry_on_busy']  # boolean option flag
     option_names = ['smc_port',
                     'api_version',
                     'smc_ssl',
                     'verify_ssl',
                     'ssl_cert_file',
+                    'retry_on_busy',
                     'timeout',
                     'domain']
 
@@ -125,7 +145,7 @@ def load_from_file(alt_filepath=None):
         'api_version': None,
         'smc_ssl': 'false',
         'verify_ssl': 'false',
-        'smc_cert_file': None,
+        'ssl_cert_file': None,
         'timeout': None,
         'domain': None},
         allow_no_value=True)
@@ -158,7 +178,7 @@ def load_from_file(alt_filepath=None):
     except configparser.NoOptionError as e:
         raise ConfigLoadError('Failed loading credentials from configuration '
                               'file: {}; {}'.format(path, e))
-    except configparser.NoSectionError as e:
+    except (configparser.NoSectionError, configparser.MissingSectionHeaderError) as e:
         raise ConfigLoadError('Failed loading credential file from: {}, check the '
                               'path and verify contents are correct.'.format(path, e))
     except IOError as e:
@@ -179,44 +199,51 @@ def transform_login(config):
     :return: dict dict of settings that can be sent into session.login
     """
     verify = True
-    if config.get('smc_ssl'):
+    if config.pop('smc_ssl', None):
         scheme = 'https'
 
-        if config.get('verify_ssl'):
+        verify = config.pop('ssl_cert_file', None)
+        if config.pop('verify_ssl', None):    
             # Get cert path to verify
-            verify = config.get('ssl_cert_file')
-            if not verify:  # Setting omitted
+            if not verify:  # Setting omitted or already False
                 verify = False
         else:
             verify = False
     else:
         scheme = 'http'
+        config.pop('verify_ssl', None)
+        config.pop('ssl_cert_file', None)
+        verify = False
 
     transformed = {}
     url = '{}://{}:{}'.format(
         scheme,
-        config.get('smc_address'),
-        config.get('smc_port'))
+        config.pop('smc_address', None),
+        config.pop('smc_port', None))
 
-    timeout = config.get('timeout')
+    timeout = config.pop('timeout', None)
     if timeout:
         try:
             timeout = int(timeout)
         except ValueError:
             timeout = None
 
-    api_version = config.get('api_version')
+    api_version = config.pop('api_version', None)
     if api_version:
         try:
-            api_version = float(api_version)
+            float(api_version)
         except ValueError:
             api_version = None
     
     transformed.update(
         url=url,
-        api_key=config.get('smc_apikey'),
+        api_key=config.pop('smc_apikey', None),
         api_version=api_version,
         verify=verify,
         timeout=timeout,
-        domain=config.get('domain'))
+        domain=config.pop('domain', None)) 
+    
+    if config:
+        transformed.update(kwargs=config) # Any remaining args
+    
     return transformed
