@@ -8,6 +8,7 @@ SSL certificates are not verified to the CA authority, need to implement for
 urllib3:
 https://urllib3.readthedocs.io/en/latest/user-guide.html#ssl
 """
+import json
 import os.path
 import collections
 import requests
@@ -17,6 +18,14 @@ from smc.api.exceptions import SMCOperationFailure, SMCConnectionError
 logger = logging.getLogger(__name__)
 
 
+class CacheEncoder(json.JSONEncoder):
+    def default(self, o):
+        try:
+            return o.data
+        except AttributeError:
+            json.JSONEncoder.default(self, o)
+
+
 class SMCAPIConnection(object):
     """
     Represents the ReST methods used to perform operations against the
@@ -31,12 +40,19 @@ class SMCAPIConnection(object):
 
     def __init__(self, session):
         self._session = session
-        self.timeout = self._session.timeout
 
+    @property
+    def timeout(self):
+        return self._session.timeout
+    
     @property
     def session(self):
         return self._session.session
 
+    @property
+    def session_domain(self):
+        return self._session.domain
+
     def send_request(self, method, request):
         """
         Send request to SMC
@@ -44,59 +60,72 @@ class SMCAPIConnection(object):
         if self.session:
             try:
                 method = method.upper() if method else ''
-
+                
                 if method == SMCAPIConnection.GET:
                     if request.filename:  # File download request
                         return self.file_download(request)
 
-                    response = self.session.get(request.href,
-                                                params=request.params,
-                                                headers=request.headers,
-                                                timeout=self.timeout)
+                    response = self.session.get(
+                        request.href,
+                        params=request.params,
+                        headers=request.headers,
+                        timeout=self.timeout)
+                    
                     response.encoding = 'utf-8'
-
-                    logger.debug(vars(response))
+                    
                     counters.update(read=1)
 
-                    if response.status_code not in (200, 304):
+                    if logger.isEnabledFor(logging.DEBUG):
+                        debug(response)
+                    
+                    if response.status_code not in (200, 204, 304):
                         raise SMCOperationFailure(response)
 
                 elif method == SMCAPIConnection.POST:
                     if request.files:  # File upload request
-                        return self.file_upload(request)
-
-                    response = self.session.post(request.href,
-                                                 # data=json.dumps(request.json),
-                                                 json=request.json,
-                                                 headers=request.headers,
-                                                 params=request.params)
+                        return self.file_upload(method, request)
+                    
+                    response = self.session.post(
+                        request.href,
+                        data=json.dumps(request.json, cls=CacheEncoder),
+                        headers=request.headers,
+                        params=request.params)
+                
                     response.encoding = 'utf-8'
 
-                    logger.debug(vars(response))
                     counters.update(create=1)
-
+                    if logger.isEnabledFor(logging.DEBUG):
+                        debug(response)
+                    
                     if response.status_code not in (200, 201, 202):
                         # 202 is asynchronous response with follower link
                         raise SMCOperationFailure(response)
 
                 elif method == SMCAPIConnection.PUT:
+                    if request.files:  # File upload request
+                        return self.file_upload(method, request)
+                    
                     # Etag should be set in request object
                     request.headers.update(Etag=request.etag)
+                    
+                    response = self.session.put(
+                        request.href,
+                        data=json.dumps(request.json, cls=CacheEncoder),
+                        params=request.params,
+                        headers=request.headers)
 
-                    response = self.session.put(request.href,
-                                                json=request.json,
-                                                params=request.params,
-                                                headers=request.headers)
-
-                    logger.debug(vars(response))
                     counters.update(update=1)
+                    
+                    if logger.isEnabledFor(logging.DEBUG):
+                        debug(response)
 
                     if response.status_code != 200:
                         raise SMCOperationFailure(response)
 
                 elif method == SMCAPIConnection.DELETE:
-                    response = self.session.delete(request.href,
-                                                   headers=request.headers)
+                    response = self.session.delete(
+                        request.href,
+                        headers=request.headers)
 
                     counters.update(delete=1)
 
@@ -110,6 +139,9 @@ class SMCAPIConnection(object):
 
                     response.encoding = 'utf-8'
 
+                    if logger.isEnabledFor(logging.DEBUG):
+                        debug(response)
+                    
                     if response.status_code not in (200, 204):
                         raise SMCOperationFailure(response)
 
@@ -123,11 +155,11 @@ class SMCAPIConnection(object):
                 raise error
             except requests.exceptions.RequestException as e:
                 raise SMCConnectionError(
-                    "Connection problem to SMC, ensure the "
-                    "API service is running and host is correct: %s, "
-                    "exiting." % e)
+                    'Connection problem to SMC, ensure the '
+                    'API service is running and host is correct: %s, '
+                    'exiting.' % e)
             else:
-                return SMCResult(response)
+                return SMCResult(response, domain=self.session_domain)
         else:
             raise SMCConnectionError(
                 "No session found. Please login to continue")
@@ -136,19 +168,19 @@ class SMCAPIConnection(object):
         """
         Called when GET request specifies a filename to retrieve.
         """
-        logger.debug(vars(request))
-        response = self.session.get(request.href,
-                                    params=request.params,
-                                    headers=request.headers,
-                                    stream=True)
+        logger.debug('Download: %s', vars(request))
+        response = self.session.get(
+            request.href,
+            params=request.params,
+            headers=request.headers,
+            stream=True)
 
         if response.status_code == 200:
-            logger.debug("Streaming to file... Content length: {}"
-                         .format(len(response.content)))
+            logger.debug('Streaming to file... Content length: %s', len(response.content))
             try:
                 path = os.path.abspath(request.filename)
-                logger.debug("Operation: {}, saving to file: {}"
-                             .format(request.href, path))
+                logger.debug('Operation: %s, saving to file: %s', request.href, path)
+    
                 with open(path, "wb") as handle:
                     for chunk in response.iter_content(chunk_size=1024):
                         if chunk:
@@ -157,31 +189,34 @@ class SMCAPIConnection(object):
             except IOError as e:
                 raise IOError('Error attempting to save to file: {}'.format(e))
 
-            result = SMCResult(response)
+            result = SMCResult(response, domain=self.session_domain)
             result.content = path
             return result
         else:
             raise SMCOperationFailure(response)
 
-    def file_upload(self, request):
+    def file_upload(self, method, request):
         """
-        Perform a file upload POST to SMC. Request should have the
+        Perform a file upload PUT/POST to SMC. Request should have the
         files attribute set which will be an open handle to the
         file that will be binary transfer.
         """
-        logger.debug(vars(request))
-        response = self.session.post(request.href,
-                                     params=request.params,
-                                     files=request.files
-                                     )
-        if response.status_code == 202:
-            logger.debug('Success sending file in elapsed time: {}'
-                         .format(response.elapsed))
-            return SMCResult(response)
+        logger.debug('Upload: %s', vars(request))
+        command = getattr(self.session, method.lower())
+        
+        response = command(
+            request.href,
+            params=request.params,
+            files=request.files)
+
+        if response.status_code in (201, 202, 204):
+            logger.debug(
+                'Success sending file in elapsed time: %s', response.elapsed)
+            return SMCResult(response, domain=self.session_domain)
 
         raise SMCOperationFailure(response)
 
-
+    
 class SMCResult(object):
     """
     SMCResult will store the return data for operations performed against the
@@ -199,12 +234,13 @@ class SMCResult(object):
     :ivar dict json: element full json
     """
 
-    def __init__(self, respobj=None, msg=None):
+    def __init__(self, respobj=None, msg=None, domain=None):
         self.etag = None
         self.href = None
         self.content = None
         self.msg = msg  # Only set in case of error
         self.code = None
+        self.domain = domain
         self.json = self._unpack_response(respobj)  # list or dict
 
     def _unpack_response(self, response):
@@ -227,11 +263,10 @@ class SMCResult(object):
                 else:
                     self.json = result  # Empty dict
                 return self.json
-            elif response.headers.get('content-type') == 'application/octet-stream':
+            elif response.headers.get('content-type') in \
+                ('application/octet-stream', 'text/plain'):
                 self.content = response.text if response.text else None
-            elif response.headers.get('content-type') == 'text/plain':
-                self.content = response.text if response.text else None
-
+            
     def __str__(self):
         sb = []
         for key in self.__dict__:
@@ -242,5 +277,21 @@ class SMCResult(object):
         return ', '.join(sb)
 
 
+def debug(response):
+    logger.debug('Request method: %s', response.request.method)
+    logger.debug('Request URL: %s', response.url)
+    logger.debug('Request headers:')
+    for k, v in response.request.headers.items():
+        logger.debug('\t%r: %r', k, v)
+    logger.debug('Request body:')
+    logger.debug('%s', response.request.body)
+    logger.debug('Response status: %s', response.status_code)
+    logger.debug('Response headers:')
+    for k, v in response.headers.items():
+        logger.debug('\t%r: %r', k, v)
+    logger.debug('Response content:')
+    logger.debug('%s', response.text)
+
+                    
 counters = collections.Counter(
     {'read': 0, 'create': 0, 'update': 0, 'delete': 0, 'cache': 0})
