@@ -1,10 +1,57 @@
 """
 Utility functions used in different areas of smc-python
 """
+import time
+import base64
+import datetime
 import smc.compat as compat
 import smc.api.exceptions
 
 
+def datetime_to_ms(dt):
+    """
+    Convert an unaware datetime object to milliseconds. This will
+    be a UTC time. The SMC stores all times in UTC and will do the
+    time conversions based on the local timezone.
+    Example of converting a datetime to milliseconds::
+    
+        utc_time = datetime.strptime("2018-06-04T00:00:00", "%Y-%m-%dT%H:%M:%S")
+        datetime_to_ms(utc_time)
+    
+    :param dt datetime: pass in python datetime object.
+    :return: value representing the datetime in milliseconds
+    :rtype: int
+    """
+    return int(time.mktime(dt.timetuple()) * 1000)
+    
+
+def datetime_from_ms(ms):
+    """
+    Convenience to return datetime from milliseconds. Return in
+    UTC time.
+    
+    :param int ms: milliseconds to convert into datetime object
+    :return: datetime from ms
+    :rtype: datetime
+    """
+    try:
+        return datetime.datetime.fromtimestamp(ms/1000.0)
+    except TypeError: # SMC version 6.2 returns in invalid format (2018-01-09T00:58:41Z)
+        return ms
+
+
+def millis_to_utc(millis):
+    """
+    Given milliseconds, convert to datetime object in UTC. This will
+    also use the systems local timezone when displaying.
+    
+    :param int millis: milliseconds
+    :return datetime using local system time conversion
+    :rtype: datetime
+    """
+    return datetime.datetime(1970, 1, 1) + datetime.timedelta(milliseconds=millis) 
+
+
 def save_to_file(filename, content):
     """
     Save content to file. Used by node initial contact but
@@ -105,9 +152,10 @@ def merge_dicts(dict1, dict2, append_lists=False):
         # append the new values onto the original list
         elif isinstance(dict2[key], list) and append_lists:
             # The value in dict1 must be a list in order to append new
-            # values onto it.
+            # values onto it. Don't add duplicates.
             if key in dict1 and isinstance(dict1[key], list):
-                dict1[key].extend(dict2[key])
+                dict1[key].extend(
+                    [k for k in dict2[key] if k not in dict1[key]])
             else:
                 dict1[key] = dict2[key]
         else:
@@ -128,6 +176,17 @@ def unicode_to_bytes(s, encoding='utf-8', errors='replace'):
     return s if isinstance(s, str) else s.encode(encoding, errors)
 
 
+def b64encode(source):
+    """
+    Base64 encoding for python 2 and 3
+    
+    :rtype: base64 content
+    """
+    if compat.PY3:
+        source = source.encode('utf-8')
+    return base64.b64encode(source).decode('utf-8')
+
+
 def bytes_to_unicode(s, encoding='utf-8', errors='replace'):
     """
     Helper to convert byte string to unicode string for user based input
