@@ -105,6 +105,12 @@ the routing node::
     >>> interface0 = engine.routing.get(0)
     >>> interface0.add_static_route(Router('tmprouter'), destination=[Group('routegroup')])
 
+When a routing gateway is added to an IPv6 network, the gateway is validated before
+adding. For example, if you have a single interface that has both an IPv4 and IPv6
+address assigned, a static route using a Router gateway with only an IPv4 address will
+only bind to the IPv4 network. In this case, you can optionally add both an IPv4 and
+IPv6 to the router element, or run this operation for each network respectively.
+
 .. seealso:: :meth:`.Routing.add_static_route`
 
 .. note:: When changing are made to a routing node, i.e. adding OSPF, BGP, Netlink's, the
@@ -141,11 +147,11 @@ class RoutingTree(SubElement):
     def __init__(self, data=None, **meta):
         super(RoutingTree, self).__init__(**meta)
         if data is not None:
-            self.data = ElementCache(**data)
+            self.data = ElementCache(data)
     
     def __iter__(self):
         for node in self.data[self.typeof]:
-            data = ElementCache(**node)
+            data = ElementCache(node)
             yield(self.__class__(
                     href=data.get_link('self'),
                     type=self.__class__.__name__,
@@ -216,7 +222,9 @@ class RoutingTree(SubElement):
         
         :rtype: str
         """
-        return self.data.get('related_element_type')
+        if 'related_element_type' in self.data:
+            return self.data.get('related_element_type')
+        return Element.from_href(self.data.get('href')).typeof # pre-6.4
 
     def as_tree(self, level=0):
         """
@@ -398,39 +406,26 @@ class Routing(RoutingTree):
             rnode = engine.routing.get(0)
             rnode.add_traffic_handler(
                 StaticNetlink('mynetlink'),
-                netlink_gw=Router('myrtr'),
+                netlink_gw=[Router('myrtr'), Host('myhost')],
                 network='172.18.1.0/24')
             
         :param StaticNetlink,Multilink netlink: netlink element
-        :param Element netlink_gw: gateway for the netlink element. Can be
-            None if no gateway is needed. Element type is typically of type
-            :class:`smc.elements.network.Router`.
+        :param list(Element) netlink_gw: list of elements that should be destinations
+            for this netlink. Typically these may be of type host, router, group, server,
+            network or engine. 
         :param str network: if network specified, only add OSPF to this network on interface
         :raises UpdateElementFailed: failure updating routing
         :raises ModificationAborted: Change must be made at the interface level
         :raises ElementNotFound: ospf area not found
-        :return: None
+        :return: Status of whether the route table was updated
+        :rtype: bool
         """
-        netlink = {
-            'href': netlink.href,
-            'level': 'gateway',
-            'routing_node': [],
-            'name': netlink.name}
-        
-        if netlink_gw:
-            netlink_gateway = {
-                'level': 'any',
-                'href': netlink_gw.href,
-                'name': netlink_gw.name}
-        
-            netlink['routing_node'].append(netlink_gateway)
-    
-        self._add_gateway_node(netlink, network)
+        routing_node_gateway = RoutingNodeGateway(netlink,
+            destinations=[] if not netlink_gw else netlink_gw)
+        return self._add_gateway_node('netlink', routing_node_gateway, network)
 
-    def add_ospf_area(self, ospf_area,
-                      communication_mode='NOT_FORCED',
-                      unicast_ref=None,
-                      network=None):
+    def add_ospf_area(self, ospf_area, ospf_interface_setting=None, network=None,
+                      communication_mode='NOT_FORCED', unicast_ref=None):
         """
         Add OSPF Area to this routing node.
 
@@ -451,40 +446,36 @@ class Routing(RoutingTree):
             interface.add_ospf_area(area)
 
         .. note:: If UNICAST is specified, you must also provide a unicast_ref
-                  to identify the remote host
+                  of element type Host to identify the remote host. If no
+                  unicast_ref is provided, this is skipped
 
         :param OSPFArea ospf_area: OSPF area instance or href
-        :param str communication_mode: NOT_FORCED|POINT_TO_POINT|PASSIVE|UNICAST
-        :param Element unicast_ref: Element used as unicast gw (required for UNICAST)
+        :param OSPFInterfaceSetting ospf_interface_setting: used to override the
+            OSPF settings for this interface (optional)
         :param str network: if network specified, only add OSPF to this network
             on interface
+        :param str communication_mode: NOT_FORCED|POINT_TO_POINT|PASSIVE|UNICAST
+        :param Element unicast_ref: Element used as unicast gw (required for UNICAST)
         :raises ModificationAborted: Change must be made at the interface level
         :raises UpdateElementFailed: failure updating routing
         :raises ElementNotFound: ospf area not found
-        :return: None
+        :return: Status of whether the route table was updated
+        :rtype: bool
         """
         communication_mode = communication_mode.upper()
-        node = {
-            'href': ospf_area.href,
-            'communication_mode': communication_mode,
-            'level': 'gateway',
-            'routing_node': [],
-            'name': ospf_area.name}
-        
-        if communication_mode == 'UNICAST':
-            # Need a destination ref, add to sub routing_node
-            node['routing_node'].append({
-                'href': unicast_ref.href,
-                'level': 'any',
-                'name': unicast_ref.name})
+        destinations=[] if not ospf_interface_setting else [ospf_interface_setting]
+        if communication_mode == 'UNICAST' and unicast_ref:
+            destinations.append(unicast_ref)
+        routing_node_gateway = RoutingNodeGateway(
+            ospf_area, communication_mode=communication_mode,
+            destinations=destinations)
+        return self._add_gateway_node('ospfv2_area', routing_node_gateway, network)
 
-        self._add_gateway_node(node, network)
-
-    def add_bgp_peering(self, bgp_peering, external_bgp_peer,
+    def add_bgp_peering(self, bgp_peering, external_bgp_peer=None,
                         network=None):
         """
         Add a BGP configuration to this routing interface. 
-        If the interface has multiple ipaddresses, all networks will receive
+        If the interface has multiple ip addresses, all networks will receive
         the BGP peering by default unless the ``network`` parameter is
         specified.
         
@@ -496,33 +487,26 @@ class Routing(RoutingTree):
                 ExternalBGPPeer('neighbor'))
 
         :param BGPPeering bgp_peering: BGP Peer element
-        :param ExternalBGPPeer external_bgp_peer: peer element or href
+        :param ExternalBGPPeer,Engine external_bgp_peer: peer element or href
         :param str network: if network specified, only add OSPF to this network
             on interface
         :raises ModificationAborted: Change must be made at the interface level
         :raises UpdateElementFailed: failed to add BGP
-        :return: None
+        :return: Status of whether the route table was updated
+        :rtype: bool
         """
-        bgp = {
-            'href': bgp_peering.href,
-            'level': 'gateway',
-            'routing_node': [],
-            'name': bgp_peering.name}
+        destination = [external_bgp_peer] if external_bgp_peer else []
+        routing_node_gateway = RoutingNodeGateway(bgp_peering,
+            destinations=destination)
+        return self._add_gateway_node('bgp_peering', routing_node_gateway, network)
 
-        external_peer = {
-            'href': external_bgp_peer.href,
-            'level': 'any',
-            'name': external_bgp_peer.name}
-        
-        bgp['routing_node'].append(external_peer)
-        
-        self._add_gateway_node(bgp, network)
-
-    def add_static_route(self, gateway, destination,
-                         network=None):
+    def add_static_route(self, gateway, destination, network=None):
         """
         Add a static route to this route table. Destination can be any element
         type supported in the routing table such as a Group of network members.
+        Since a static route gateway needs to be on the same network as the
+        interface, provide a value for `network` if an interface has multiple
+        addresses on different networks.
         ::
 
             >>> engine = Engine('ve-1')
@@ -536,22 +520,13 @@ class Routing(RoutingTree):
         :type destination: list(Host, Router, ..)
         :raises ModificationAborted: Change must be made at the interface level
         :raises UpdateElementFailed: failure to update routing table
-        :return: None
+        :return: Status of whether the route table was updated
+        :rtype: bool
         """
-        route = {
-            'href': gateway.href,
-            'level': 'gateway',
-            'routing_node': [],
-            'name': gateway.name}
-        
-        for dest in destination:
-            route['routing_node'].append({
-                'href': dest.href,
-                'level': 'any',
-                'name': dest.name})            
-        
-        self._add_gateway_node(route, network)
-    
+        routing_node_gateway = RoutingNodeGateway(gateway,
+            destinations=destination)     
+        return self._add_gateway_node('router', routing_node_gateway, network)
+
     def add_dynamic_gateway(self, networks):
         """
         A dynamic gateway object creates a router object that is 
@@ -567,48 +542,131 @@ class Routing(RoutingTree):
             this DHCP gateway
         :raises ModificationAborted: Change must be made at the interface level
         :raises UpdateElementFailed: failure to update routing table
-        :return: None
+        :return: Status of whether the route table was updated
+        :rtype: bool
         """
-        route = {
-            'dynamic_classid': 'gateway',
-            'level': 'gateway',
-            'routing_node': []}
-        
-        for network in networks:
-            route['routing_node'].append({
-                'href': network.href,
-                'level': 'any',
-                'name': network.name})
-
-        self._add_gateway_node(route)
+        routing_node_gateway = RoutingNodeGateway(dynamic_classid='gateway',
+            destinations=networks or [])
+        return self._add_gateway_node('dynamic_netlink', routing_node_gateway)
     
-    def _add_gateway_node(self, element, network=None):
+    def _add_gateway_node_on_tunnel(self, routing_node_gateway):
         """
-        Bind the pre-configured element to the interface level
-        routing node. It will be saved back at the interface. This
-        should be called when level == 'interface'.
+        Add a gateway node on a tunnel interface. Tunnel interface elements
+        are attached to the interface level and not directly nested under
+        the networks node.
+        
+        :param RouteNodeGateway routing_node_gateway: routing node gateway instance
+        :return: Whether a change was made or not
+        :rtype: bool
+        """
+        modified = False
+        peering = [next_hop for next_hop in self
+            if next_hop.routing_node_element == routing_node_gateway.routing_node_element]
+        if not peering:
+            self.data.setdefault('routing_node', []).append( 
+                routing_node_gateway)
+            modified = True
+        # Have peering
+        else:
+            peers = [node.routing_node_element for peer in peering
+                for node in peer]
+            for destination in routing_node_gateway.destinations:
+                if destination not in peers:
+                    peering[0].data.setdefault('routing_node', []).append(
+                        {'level': 'any', 'href': destination.href,
+                         'name': destination.name})
+                    modified = True
+        if modified:
+            self.update()
+        return modified
+
+    def _add_gateway_node(self, gw_type, routing_node_gateway, network=None):
+        """
+        Add a gateway node to existing routing tree. Gateways are only added if
+        they do not already exist. If they do exist, check the destinations of
+        the existing gateway and add destinations that are not already there.
+        A current limitation is that if a gateway doesn't exist and the
+        destinations specified do not have IP addresses that are valid, they
+        are still added (i.e. IPv4 gateway with IPv6 destination is considered
+        invalid).
+        
+        :param Routing self: the routing node, should be the interface routing node
+        :param str gw_type: type of gateway, i.e. netlink, ospfv2_area, etc
+        :param RoutingNodeGateway route_node_gateway: gateway element
+        :param str network: network to bind to. If none, all networks
+        :return: Whether a change was made or not
+        :rtype: bool
         """
         if self.level != 'interface':
             raise ModificationAborted('You must make this change from the '
                 'interface routing level. Current node: {}'.format(self))
         
-        node_added = False    
-        for networks in self:
-            if networks.dynamic_nicid: # DHCP interface
-                networks.data['routing_node'].append(element)
-                node_added = True
+        if self.related_element_type == 'tunnel_interface':
+            return self._add_gateway_node_on_tunnel(routing_node_gateway)
+        
+        # Find any existing gateways
+        routing_node = list(gateway_by_type(self, type=gw_type, on_network=network))
+        
+        _networks = [netwk for netwk in self if netwk.ip == network] if network is \
+            not None else list(self)
+        
+        # Routing Node Gateway to add as Element
+        gateway_element_type = routing_node_gateway.routing_node_element
+        
+        modified = False
+        for network in _networks:
+            # Short circuit for dynamic interfaces
+            if getattr(network, 'dynamic_nicid', None):
+                network.data.setdefault('routing_node', []).append(
+                        routing_node_gateway)
+                modified = True
                 break
-            if len(networks.ip.split(':')) == 1:  # Skip IPv6
-                if network is not None:  # Only place on specific network
-                    if networks.ip == network:
-                        networks.data['routing_node'].append(element)
-                        node_added = True
-                else: # Place on all networks
-                    networks.data['routing_node'].append(element)
-                    node_added = True
-        if node_added:
+            
+            # Used for comparison to 
+            this_network_node = network.routing_node_element
+            
+            if routing_node and any(netwk for _intf, netwk, gw in routing_node
+                if netwk.routing_node_element == this_network_node and
+                gateway_element_type == gw.routing_node_element):
+                
+                # A gateway exists on this network
+                for gw in network:
+                    if gw.routing_node_element == gateway_element_type:
+                        existing_dests = [node.routing_node_element for node in gw]
+                        for destination in routing_node_gateway.destinations:
+                            is_valid_destination = False
+                            if destination not in existing_dests:
+                                dest_ipv4, dest_ipv6 = _which_ip_protocol(destination)
+                                if len(network.ip.split(':')) > 1: # IPv6
+                                    if dest_ipv6:
+                                        is_valid_destination = True
+                                else:
+                                    if dest_ipv4:
+                                        is_valid_destination = True
+                
+                                if is_valid_destination:
+                                    gw.data.setdefault('routing_node', []).append(
+                                        {'level': 'any', 'href': destination.href,
+                                         'name': destination.name})
+                                    modified = True
+        
+            else: # Gateway doesn't exist
+                gw_ipv4, gw_ipv6 = _which_ip_protocol(gateway_element_type) # ipv4, ipv6 or both
+                if len(network.ip.split(':')) > 1:
+                    if gw_ipv6:
+                        network.data.setdefault('routing_node', []).append(
+                            routing_node_gateway)
+                        modified = True
+                else: # IPv4
+                    if gw_ipv4:
+                        network.data.setdefault('routing_node', []).append(
+                            routing_node_gateway) 
+                        modified = True
+
+        if modified:
             self.update()
-                    
+        return modified    
+                 
     def remove_route_gateway(self, element, network=None):
         """
         Remove a route element by href or Element. Use this if you want to
@@ -630,18 +688,48 @@ class Routing(RoutingTree):
             network on interface
         :raises ModificationAborted: Change must be made at the interface level
         :raises UpdateElementFailed: failure to update routing table
-        :return: None
+        :return: Status of whether the entry was removed (i.e. or not found)
+        :rtype: bool
         """
-        if self.level != 'interface':
+        if self.level not in ('interface'):
             raise ModificationAborted('You must make this change from the '
                 'interface routing level. Current node: {}'.format(self))
-                
+        
+        node_changed = False
         element = element_resolver(element)
         for network in self:
+            # Tunnel Interface binds gateways to the interface
+            if network.level == 'gateway' and network.data.get('href') == element:
+                network.delete()
+                node_changed = True
+                break
             for gateway in network:
                 if gateway.data.get('href') == element:
                     gateway.delete()
-    
+                    node_changed = True
+        return node_changed
+
+
+class RoutingNodeGateway(Routing):
+    def __init__(self, element=None, level='gateway', **kwargs):
+        self.destinations = kwargs.pop('destinations', [])
+        self.data = ElementCache(kwargs)
+        self.data.update(
+            level=level,
+            routing_node=[])
+
+        if element:
+            self.data.update(
+                href=element.href,
+                name=element.name)
+                #related_element_type=element.typeof)
+        
+        for destination in self.destinations:
+            self.data['routing_node'].append(
+                {'href': destination.href,
+                 'name': destination.name,
+                 'level': 'any'})
+
 
 class Antispoofing(RoutingTree):
     """
@@ -682,34 +770,67 @@ class Antispoofing(RoutingTree):
         :rtype: str
         """
         return self.data.get('validity')
-
-    def add(self, entry):
+    
+    def add(self, element):
         """
         Add an entry to this antispoofing node level.
         Entry can be either href or network elements specified
         in :py:class:`smc.elements.network`
         ::
 
-            for entry in engine.antispoofing.all():
-                if entry.name == 'Interface 0':
-                    entry.add(Network('network-10.1.2.0/24'))
+            if0 = engine.antispoofing.get(0)
+            if0.add(Network('foonet'))
 
-        :param Element entry: entry to add, i.e. Network('mynetwork'), Host(..)
-        :return: None
+        :param Element element: entry to add, i.e. Network('mynetwork'), Host(..)
         :raises CreateElementFailed: failed adding entry
         :raises ElementNotFound: element entry specified not in SMC
+        :return: whether entry was added
+        :rtype: bool
         """
-        node = {
-            'antispoofing_node': [],
-            'auto_generated': 'false',
-            'href': entry.href,
-            'level': self.level,
-            'validity': 'enable',
-            'name': entry.name}
-
-        self.data['antispoofing_node'].append(node)
-        self.update()
+        if self.level == 'interface':
+            for network in self:
+                if from_meta(network) == element:
+                    return False
+        
+            self.data['antispoofing_node'].append({
+                'antispoofing_node': [],
+                'auto_generated': 'false',
+                'href': element.href,
+                'level': self.level,
+                'validity': 'enable',
+                'name': element.name})
 
+            self.update()
+            return True
+        return False
+    
+    def __len__(self):
+        return len(self.data.get('antispoofing_node', []))
+    
+    def remove(self, element):
+        """
+        Remove a specific user added element from the antispoofing tables of
+        a given interface. This will not remove autogenerated or system level
+        entries.
+        
+        :param Element element: element to remove
+        :return: remove element if it exists and return bool
+        :rtype: bool
+        """
+        if self.level == 'interface':
+            len_before_change = len(self)
+            _nodes = []
+            for network in self:
+                if from_meta(network) != element:
+                    _nodes.append(network.data)
+                else:
+                    if network.autogenerated: # Make sure it was user added
+                        _nodes.append(network.data)
+            if len(_nodes) != len_before_change:
+                self.data['antispoofing_node'] = _nodes
+                self.update()
+                return True
+        return False
 
 def from_meta(node):
     """
@@ -720,7 +841,7 @@ def from_meta(node):
     :rtype: Element
     """
     # Version SMC < 6.4
-    if not node.related_element_type:
+    if 'related_element_type' not in node.data:
         return Element.from_href(
             node.data.get('href'))
     
@@ -749,11 +870,19 @@ def route_level(root, level):
     return routing_node
 
 
-def gateway_by_type(self, type=None):  # @ReservedAssignment
+def gateway_by_type(self, type=None, on_network=None):  # @ReservedAssignment
     """
     Return gateways for the specified node. You can also
     specify type to find only gateways of a specific type.
     Valid types are: bgp_peering, netlink, ospfv2_area.
+    
+    :param RoutingNode self: the routing node to check
+    :param str type: bgp_peering, netlink, ospfv2_area
+    :param str on_network: if network is specified, should be CIDR and
+        specifies a filter to only return gateways on that network when
+        an interface has multiple
+    :return: tuple of RoutingNode(interface,network,gateway)
+    :rtype: list
     """
     gateways = route_level(self, 'gateway')
     if not type:
@@ -764,10 +893,70 @@ def gateway_by_type(self, type=None):  # @ReservedAssignment
             #TODO: Change to type == node.related_element_type when
             # only supporting SMC >= 6.4
             if type == node.routing_node_element.typeof:
-                network = node._parent
-                interface = network._parent
-                yield (interface, network, node)
-                    
+                # If the parent is level interface, this is a tunnel interface
+                # where the gateway is bound to interface versus network
+                parent = node._parent
+                if parent.level == 'interface':
+                    interface = parent
+                    network = None
+                else:
+                    network = parent
+                    interface = network._parent
+                
+                if on_network is not None:
+                    if network and network.ip == on_network:
+                        yield (interface, network, node)
+                else:
+                    yield (interface, network, node)
+
+
+def _which_ip_protocol(element):
+    """
+    Validate the protocol addresses for the element. Most elements can
+    have an IPv4 or IPv6 address assigned on the same element. This
+    allows elements to be validated and placed on the right network.
+    
+    :return: boolean tuple
+    :rtype: tuple(ipv4, ipv6)
+    """
+    try:
+        if element.typeof in ('host', 'router'):
+            return getattr(element, 'address', False), getattr(element, 'ipv6_address', False)
+        elif element.typeof == 'netlink':
+            gateway = element.gateway
+            if gateway.typeof == 'router':
+                return getattr(gateway, 'address', False), getattr(gateway, 'ipv6_address', False)
+            # It's an engine, return true
+        elif element.typeof == 'network':
+            return getattr(element, 'ipv4_network', False), getattr(element, 'ipv6_network', False)
+    except AttributeError:
+        pass
+    # Always return true so that the calling function assumes the element
+    # is valid for the routing node. This could fail when submitting but
+    # we don't want to prevent adding elements yet since this could change
+    return True, True
+
+
+def del_invalid_routes(engine, nicids):
+    """
+    Helper method to run through and delete any routes that are tagged
+    as invalid or to_delete by a list of nicids. Since we could have a
+    list of routes, iterate from top level engine routing node to avoid
+    fetch exceptions. Route list should be a list of nicids as str.
+    
+    :param list nicids: list of nicids
+    :raises DeleteElementFailed: delete element failed with reason
+    """
+    nicids = map(str, nicids)
+    for interface in engine.routing:
+        if interface.nicid in nicids:
+            if getattr(interface, 'to_delete', False): # Delete the invalid interface
+                interface.delete()
+                continue
+        for network in interface:
+            if getattr(network, 'invalid', False) or \
+                getattr(network, 'to_delete', False):
+                network.delete()
 
 
 route = collections.namedtuple('Route',
