@@ -1,11 +1,9 @@
 """
 Module representing network elements used within the SMC
 """
-from smc.base.model import Element, ElementCreator, prepared_request,\
-    SimpleElement
+from smc.base.model import Element, ElementCreator
 from smc.api.exceptions import MissingRequiredInput, CreateElementFailed,\
-    ElementNotFound
-from smc.api.common import fetch_json_by_href
+    ElementNotFound, FetchElementFailed
 
 
 class Host(Element):
@@ -15,14 +13,14 @@ class Host(Element):
     Create a host element with ipv4::
 
         Host.create(name='myhost', address='1.1.1.1',
-                    secondary_ip=['1.1.1.2'],
+                    secondary=['1.1.1.2'],
                     comment='some comment for my host')
 
     Create a host element with ipv6 and secondary ipv4 address::
 
         Host.create(name='mixedhost',
                     ipv6_address='2001:cdba::3257:9652',
-                    secondary_ip=['1.1.1.1'])
+                    secondary=['1.1.1.1'])
     
     Available attributes:
     
@@ -32,9 +30,6 @@ class Host(Element):
     """
     typeof = 'host'
 
-    def __init__(self, name, **meta):
-        super(Host, self).__init__(name, **meta)
-        
     @classmethod
     def create(cls, name, address=None, ipv6_address=None,
                secondary=None, comment=None):
@@ -76,6 +71,7 @@ class Host(Element):
             secondary=address,
             append_lists=append_lists)
 
+
 class AddressRange(Element):
     """
     Class representing a IpRange object used in access rules
@@ -90,9 +86,6 @@ class AddressRange(Element):
         '10.10.10.1-10.10.10.10'
     """
     typeof = 'address_range'
-    
-    def __init__(self, name, **meta):
-        super(AddressRange, self).__init__(name, **meta)
         
     @classmethod
     def create(cls, name, ip_range, comment=None):
@@ -123,30 +116,27 @@ class Router(Element):
 
     Create a router element with ipv6 address::
 
-        Host.create(name='mixedhost',
-                    ipv6_address='2001:cdba::3257:9652')
+        Router.create(name='mixedhost',
+                      ipv6_address='2001:cdba::3257:9652')
     
     Available attributes:
     
     :ivar str address: IPv4 address for this router
     :ivar str ipv6_address: IPv6 address for this router
-    :ivar list secondary_ip: list of additional IP's for this router
+    :ivar list secondary: list of additional IP's for this router
     """
     typeof = 'router'
 
-    def __init__(self, name, **meta):
-        super(Router, self).__init__(name, **meta)
-        
     @classmethod
     def create(cls, name, address=None, ipv6_address=None,
-               secondary_ip=None, comment=None):
+               secondary=None, comment=None):
         """
         Create the router element
 
         :param str name: Name of element
         :param str address: ip address of host object (optional if ipv6)
         :param str ipv6_address: ipv6 address (optional if ipv4)
-        :param str secondary_ip: secondary ip address (optional)
+        :param list secondary: secondary ip address (optional)
         :param str comment: comment (optional)
         :raises CreateElementFailed: element creation failed with reason
         :return: instance with meta
@@ -156,7 +146,7 @@ class Router(Element):
         """
         address = address if address else None
         ipv6_address = ipv6_address if ipv6_address else None
-        secondary = [] if secondary_ip is None else secondary_ip
+        secondary = [] if secondary is None else secondary
         json = {'name': name,
                 'address': address,
                 'ipv6_address': ipv6_address,
@@ -169,7 +159,12 @@ class Router(Element):
 class Network(Element):
     """
     Class representing a Network object used in access rules
-    Network format should be CIDR based.
+    Network format should be CIDR based. It is recommended that when
+    creating the network element, you use a naming convention that
+    includes the network cidr in the name, such as 'network-1.1.1.0/24'.
+    This will simplify searches later and workaround the restriction
+    that searches with '/' and '-' only match on the name field and
+    not an actual attribute value.
 
     Create an ipv4 network element::
 
@@ -186,9 +181,6 @@ class Network(Element):
     """
     typeof = 'network'
 
-    def __init__(self, name, **meta):
-        super(Network, self).__init__(name, **meta)
-        
     @classmethod
     def create(cls, name, ipv4_network=None, ipv6_network=None,
                comment=None):
@@ -214,7 +206,7 @@ class Network(Element):
 
         return ElementCreator(cls, json)
 
-
+    
 class DomainName(Element):
     """
     Represents a domain name used as FQDN in policy
@@ -227,9 +219,6 @@ class DomainName(Element):
     """
     typeof = 'domain_name'
 
-    def __init__(self, name, **meta):
-        super(DomainName, self).__init__(name, **meta)
-        
     @classmethod
     def create(cls, name, comment=None):
         """
@@ -269,9 +258,6 @@ class Expression(Element):
     """
     typeof = 'expression'
 
-    def __init__(self, name, **meta):
-        super(Expression, self).__init__(name, **meta)
-        
     @staticmethod
     def build_sub_expression(name, ne_ref=None, operator='union'):
         """
@@ -337,9 +323,6 @@ class URLListApplication(Element):
     """
     typeof = 'url_list_application'
 
-    def __init__(self, name, **meta):
-        super(URLListApplication, self).__init__(name, **meta)
-        
     @classmethod
     def create(cls, name, url_entry, comment=None):
         """
@@ -384,24 +367,26 @@ class IPList(Element):
 
     Example of downloading the IPList in text format::
 
-        >>> iplist = list(Search('ip_list').objects.filter('mylist'))
+        >>> iplist = list(IPList.objects.filter('mylist'))
         >>> print(iplist)
         [IPList(name=mylist)]
         >>> iplist[0].download(filename='iplist.txt', as_type='txt')
 
     Example of uploading an IPList as a zip file::
 
-        >>> iplist = list(Search('ip_list').objects.filter('mylist'))
+        >>> iplist = list(IPList.objects.filter('mylist'))
         >>> print(iplist)
         [IPList(name=mylist)]
         iplist[0].upload(filename='/path/to/iplist.zip')
 
+    Upload an IPList using json format::
+    
+        >>> iplist = IPList('mylist')
+        >>> iplist.upload(json={'ip': ['4.4.4.4']}, as_type='json')
+    
     """
     typeof = 'ip_list'
 
-    def __init__(self, name, **meta):
-        super(IPList, self).__init__(name, **meta)
-        
     def download(self, filename=None, as_type='zip'):
         """
         Download the IPList. List format can be either zip, text or
@@ -424,12 +409,15 @@ class IPList(Element):
                 headers = {'accept': 'text/plain'}
             elif as_type == 'json':
                 headers = {'accept': 'application/json'}
-
-            prepared_request(
-                href=self.data.get_link('ip_address_list'),
+        
+            result = self.make_request(
+                FetchElementFailed,
+                raw_result=True,
+                resource='ip_address_list',
                 filename=filename,
-                headers=headers
-            ).read()
+                headers=headers)
+        
+            return result.json if as_type == 'json' else result.content
 
     def upload(self, filename=None, json=None, as_type='zip'):
         """
@@ -458,13 +446,63 @@ class IPList(Element):
         elif as_type == 'txt':
             params = {'format': 'txt'}
 
-        prepared_request(
+        self.make_request(
             CreateElementFailed,
-            href=self.data.get_link('ip_address_list'),
+            method='create',
+            resource='ip_address_list',
             headers=headers, files=files, json=json,
-            params=params
-        ).create()
+            params=params)
 
+    @classmethod
+    def update_or_create(cls, append_lists=True, with_status=False, **kwargs):
+        """
+        Update or create an IPList.
+        
+        :param bool append_lists: append to existing IP List
+        :param dict kwargs: provide at minimum the name attribute
+            and optionally match the create constructor values
+        :raises FetchElementFailed: Reason for retrieval failure
+        """
+        was_created, was_modified = False, False 
+        element = None
+        try: 
+            element = cls.get(kwargs.get('name')) 
+            if append_lists:
+                iplist = element.iplist
+                diff = [i for i in kwargs.get('iplist', []) if i not in iplist]
+                if diff:
+                    iplist.extend(diff)
+                else:
+                    iplist = []
+            else:
+                iplist = kwargs.get('iplist', [])
+            
+            if iplist:
+                element.upload(json={'ip': iplist}, as_type='json')
+                was_modified = True
+    
+        except ElementNotFound:
+            element = cls.create( 
+                kwargs.get('name'), 
+                iplist = kwargs.get('iplist', []))
+            was_created = True
+
+        if with_status: 
+            return element, was_modified, was_created 
+        return element 
+                
+    @property
+    def iplist(self):
+        """
+        Return a list representation of this IPList. This is not
+        a recommended function if the list is extremely large.
+        In that case use the download function in zip format.
+        
+        :raises FetchElementFailed: Reason for retrieval failure
+        :rtype: list
+        """
+        return self.download(as_type='json').get('ip', [])
+    
     @classmethod
     def create(cls, name, iplist=None, comment=None):
         """
@@ -479,17 +517,17 @@ class IPList(Element):
         :return: instance with meta
         :rtype: IPList
         """
-        json = {'name': name,
-                'comment': comment}
+        json = {'name': name, 'comment': comment}
         result = ElementCreator(cls, json)
         if result and iplist is not None:
             element = IPList(name)
 
-            prepared_request(
+            element.make_request(
                 CreateElementFailed,
-                href=element.data.get_link('ip_address_list'),
-                json={'ip': iplist}
-            ).create()
+                method='create',
+                resource='ip_address_list',
+                json={'ip': iplist})
+    
         return result
 
 
@@ -506,9 +544,6 @@ class Zone(Element):
     """
     typeof = 'interface_zone'
     
-    def __init__(self, name, **meta):
-        super(Zone, self).__init__(name, **meta)
-        
     @classmethod
     def create(cls, name, comment=None):
         """
@@ -543,6 +578,7 @@ class IPCountryGroup(Element):
     """
     typeof = 'ip_country_group'
 
+
 class Alias(Element):
     """
     Aliases are adaptive objects that represent a single
@@ -554,28 +590,44 @@ class Alias(Element):
     or loading directly if you know the alias name:
     ::
 
-        >>> list(Search('alias').objects.all())
+        >>> list(Alias.objects.all())
         [Alias(name=$$ Interface ID 46.net), Alias(name=$$ Interface ID 45.net), etc]
 
         >>> from smc.elements.network import Alias
         >>> alias = Alias('$$ Interface ID 0.ip')
         >>> print(alias)
         Alias(name=$$ Interface ID 0.ip)
+        
+    Resolve this to a specific engine::
+    
+        >>> alias = Alias('$$ Interface ID 0.ip')
+        >>> alias.resolve('myfirewall')
+        [u'10.10.0.1']  
     """
     typeof = 'alias'
 
     def __init__(self, name, **meta):
         super(Alias, self).__init__(name, **meta)
-        self.resolved_value = []
+        self.resolved_value = [] #: resolved value for alias
 
     @classmethod
-    def load(cls, data):
-        href = data.get('alias_ref')
-        result = fetch_json_by_href(href)
-        alias = Alias(result.json.get('name'), href=href)
-        alias.data = SimpleElement(etag=result.etag, **result.json)
-        alias.resolved_value = data.get('resolved_value')
-        return alias
+    def from_engine(cls, data, alias_list):
+        """
+        Return an alias for the engine. The data is dict provided
+        when calling engine.alias_resolving(). The alias list is
+        the list of aliases pre-fetched from Alias.objects.all().
+        This will return an Alias element by taking the alias_ref
+        and finding the name in the alias list.
+        
+        :rtype: Alias
+        """
+        for alias in alias_list:
+            href = data.get('alias_ref')
+            if alias.href == href:
+                _alias = Alias(alias.name, href=href)
+                _alias.resolved_value = data.get('resolved_value')
+                _alias.typeof = alias._meta.type
+                return _alias
 
     def resolve(self, engine):
         """
@@ -593,11 +645,10 @@ class Alias(Element):
         :rtype: list
         """
         if not self.resolved_value:
-            result = prepared_request(
+            result = self.make_request(
                 ElementNotFound,
-                href=self.data.get_link('resolve'),
-                params={'for': engine}
-            ).read()
+                href=self.get_relation('resolve'),
+                params={'for': engine})
 
-            self.resolved_value = result.json.get('resolved_value')
+            self.resolved_value = result.get('resolved_value')
         return self.resolved_value
