@@ -14,7 +14,7 @@ To get an existing policy::
 Or through collections::
 
     >>> from smc.policy.layer2 import Layer2Policy
-    >>> list(Search('layer2_policy').objects.all())
+    >>> list(Layer2Policy.objects.all())
     [Layer2Policy(name=MyLayer2Policy)]
     
 To create a new policy, use::
@@ -53,7 +53,7 @@ from smc.api.exceptions import ElementNotFound, LoadPolicyFailed,\
     CreatePolicyFailed, CreateElementFailed
 from smc.policy.policy import Policy
 from smc.policy.rule import IPv4Layer2Rule, EthernetRule
-from smc.base.collection import create_collection
+from smc.base.collection import rule_collection
 
 
 class Layer2Rule(object):
@@ -67,11 +67,10 @@ class Layer2Rule(object):
         """ 
         Layer2 Firewall access rule
 
-        :return: collection of :class:`smc.policy.rule.IPv4Layer2Rule`
-        :rtype: SubElementCollection
+        :rtype: rule_collection(IPv4Layer2Rule)
         """
-        return create_collection(
-            self.data.get_link('layer2_ipv4_access_rules'),
+        return rule_collection(
+            self.get_relation('layer2_ipv4_access_rules'),
             IPv4Layer2Rule)
 
     @property
@@ -88,11 +87,10 @@ class Layer2Rule(object):
         """
         Layer 2 Ethernet access rule
 
-        :return: collection of :class:`smc.policy.rule.EthernetRule`
-        :rtype: SubElementCollection
+        :rtype: rule_collection(EthernetRule)
         """
-        return create_collection(
-            self.data.get_link('layer2_ethernet_rules'),
+        return rule_collection(
+            self.get_relation('layer2_ethernet_rules'),
             EthernetRule)
 
 
@@ -113,10 +111,6 @@ class Layer2Policy(Layer2Rule, Policy):
     """
     typeof = 'layer2_policy'
 
-    def __init__(self, name, **meta):
-        super(Layer2Policy, self).__init__(name, **meta)
-        pass
-
     @classmethod
     def create(cls, name, template):
         """ 
@@ -153,8 +147,7 @@ class Layer2Policy(Layer2Rule, Policy):
         try:
             return ElementCreator(cls, json)
         except CreateElementFailed as err:
-            raise CreatePolicyFailed(
-                'Failed to create firewall policy: {}'.format(err))
+            raise CreatePolicyFailed(err)
 
 
 class Layer2TemplatePolicy(Layer2Rule, Policy):
@@ -175,10 +168,4 @@ class Layer2TemplatePolicy(Layer2Rule, Policy):
     """
     typeof = 'layer2_template_policy'
 
-    def __init__(self, name, **meta):
-        super(Layer2TemplatePolicy, self).__init__(name, **meta)
-        pass
-
-    def export(self): pass  # Not supported on the template
-
     def upload(self): pass  # Not supported on the template
