@@ -13,7 +13,7 @@ To get an existing policy::
 
 Or through collections::
 
-    >>> list(Search('fw_policy').objects.all())
+    >>> list(FirewallPolicy.objects.all())
     [FirewallPolicy(name=Standard Firewall Policy with Inspection), FirewallPolicy(name=Layer 3 Virtual FW Policy)]
 
 To create a new policy, use::
@@ -42,9 +42,9 @@ from smc.api.exceptions import CreatePolicyFailed, ElementNotFound, LoadPolicyFa
 from smc.policy.policy import Policy
 from smc.policy.rule import IPv4Rule, IPv6Rule
 from smc.policy.rule_nat import IPv4NATRule, IPv6NATRule
-from smc.base.collection import create_collection
-
+from smc.base.collection import rule_collection
 
+                
 class FirewallRule(object):
     """
     Encapsulates all references to firewall rule related entry
@@ -56,11 +56,10 @@ class FirewallRule(object):
         """
         IPv4 rule entry point
 
-        :return: collection of :class:`smc.policy.rule.IPv4Rule`
-        :rtype: SubElementCollection
+        :rtype: rule_collection(IPv4Rule)
         """
-        return create_collection(
-            self.data.get_link('fw_ipv4_access_rules'),
+        return rule_collection(
+            self.get_relation('fw_ipv4_access_rules'),
             IPv4Rule)
 
     @property
@@ -68,11 +67,10 @@ class FirewallRule(object):
         """
         IPv4NAT Rule entry point
 
-        :return: collection of :class:`smc.policy.rule_nat.IPv4NATRule`
-        :rtype: SubElementCollection
+        :rtype: rule_collection(IPv4NATRule)
         """
-        return create_collection(
-            self.data.get_link('fw_ipv4_nat_rules'),
+        return rule_collection(
+            self.get_relation('fw_ipv4_nat_rules'),
             IPv4NATRule)
 
     @property
@@ -80,11 +78,10 @@ class FirewallRule(object):
         """
         IPv6 Rule entry point
 
-        :return: collection of :class:`smc.policy.rule.IPv6Rule`
-        :rtype: SubElementCollection
+        :rtype: rule_collection(IPv6Rule)
         """
-        return create_collection(
-            self.data.get_link('fw_ipv6_access_rules'),
+        return rule_collection(
+            self.get_relation('fw_ipv6_access_rules'),
             IPv6Rule)
 
     @property
@@ -92,11 +89,10 @@ class FirewallRule(object):
         """
         IPv6NAT Rule entry point
 
-        :return: collection of :class:`smc.policy.rule.IPv6NATRule`
-        :rtype: SubElementCollection
+        :rtype: rule_collection(IPv6NATRule)
         """
-        return create_collection(
-            self.data.get_link('fw_ipv6_nat_rules'),
+        return rule_collection(
+            self.get_relation('fw_ipv6_nat_rules'),
             IPv6NATRule)
 
 
@@ -120,10 +116,6 @@ class FirewallPolicy(FirewallRule, Policy):
     """
     typeof = 'fw_policy'
 
-    def __init__(self, name, **meta):
-        super(FirewallPolicy, self).__init__(name, **meta)
-        pass
-
     @classmethod
     def create(cls, name, template='Firewall Inspection Template'):
         """
@@ -157,8 +149,7 @@ class FirewallPolicy(FirewallRule, Policy):
         try:
             return ElementCreator(cls, json)
         except CreateElementFailed as err:
-            raise CreatePolicyFailed(
-                'Failed to create firewall policy: {}'.format(err))
+            raise CreatePolicyFailed(err)
 
 
 class FirewallSubPolicy(Policy):
@@ -182,10 +173,6 @@ class FirewallSubPolicy(Policy):
     """
     typeof = 'sub_ipv4_fw_policy'
 
-    def __init__(self, name, **meta):
-        super(FirewallSubPolicy, self).__init__(name, **meta)
-        pass
-
     @classmethod
     def create(cls, name):
         """
@@ -204,11 +191,10 @@ class FirewallSubPolicy(Policy):
         """
         IPv4 rule entry point
 
-        :return: collection of :class:`smc.policy.rule.IPv4Rule`
-        :rtype: SubElementCollection
+        :rtype: rule_collection(IPv4Rule)
         """
-        return create_collection(
-            self.data.get_link('fw_ipv4_access_rules'),
+        return rule_collection(
+            self.get_relation('fw_ipv4_access_rules'),
             IPv4Rule)
 
 
@@ -230,10 +216,4 @@ class FirewallTemplatePolicy(FirewallRule, Policy):
     """
     typeof = 'fw_template_policy'
 
-    def __init__(self, name, **meta):
-        super(FirewallTemplatePolicy, self).__init__(name, **meta)
-        pass
-
-    def export(self): pass  # Not supported on the template
-
     def upload(self): pass  # Not supported on the template
