@@ -5,7 +5,7 @@ Rule module is a base class for all access control and NAT rules.
 
     Policy (base)
           |
-    Layer3FirewallPolicy -----> fw_ipv4_access_rules
+    FirewallPolicy ------------> fw_ipv4_access_rules
                                         |
                                         |
                               IPv4Rule / IPv4NATRule (smc.policy.rule.Rule)
@@ -70,14 +70,14 @@ Examples of rule operations::
     IPv4Rule(name=discard at bottom) discard at bottom discard
 
 """
-from smc.base.model import Element, SubElement, prepared_request
+from smc.base.model import Element, SubElement, SubElementCreator
 from smc.elements.other import LogicalInterface
-from smc.vpn.policy import VPNPolicy
 from smc.api.exceptions import ElementNotFound, MissingRequiredInput,\
-    CreateRuleFailed, PolicyCommandFailed, FetchElementFailed
+    CreateRuleFailed, PolicyCommandFailed
 from smc.policy.rule_elements import Action, LogOptions, Destination, Source,\
     Service, AuthenticationOptions, TimeRange
 from smc.base.util import element_resolver
+from smc.base.decorators import cacheable_resource
 
 
 class Rule(object):
@@ -94,31 +94,66 @@ class Rule(object):
         """
         Name attribute of rule element
         """
-        return self._meta.name
+        return self._meta.name if self._meta.name else \
+            'Rule @%s' % self.tag
 
-    def add_after(self):
-        pass
+    def move_rule_after(self, other_rule):
+        """
+        Add this rule after another. This process will make a copy of
+        the existing rule and add after the specified rule. If this
+        raises an exception, processing is stopped. Otherwise the original
+        rule is then deleted.
+        You must re-retrieve the new element after running this operation
+        as new references will be created.
+        
+        :param other_rule Rule: rule where this rule will be positioned after
+        :raises CreateRuleFailed: failed to duplicate this rule, no move
+            is made
+        """
+        self.make_request(
+            CreateRuleFailed,
+            href=other_rule.get_relation('add_after'),
+            method='create',
+            json=self)
+        self.delete()
 
-    def add_before(self):
-        pass
-
-    @property
+    def move_rule_before(self, other_rule):
+        """
+        Move this rule after another. This process will make a copy of
+        the existing rule and add after the specified rule. If this
+        raises an exception, processing is stopped. Otherwise the original
+        rule is then deleted.
+        You must re-retrieve the new element after running this operation
+        as new references will be created.
+        
+        :param other_rule Rule: rule where this rule will be positioned before
+        :raises CreateRuleFailed: failed to duplicate this rule, no move
+            is made
+        """
+        self.make_request(
+            CreateRuleFailed,
+            href=other_rule.get_relation('add_before'),
+            method='create',
+            json=self)
+        self.delete()
+    
+    @cacheable_resource
     def action(self):
         """
         Action for this rule. 
 
-        :return: :py:class:`smc.policy.rule_elements.Action`
+        :rtype: Action
         """
-        return Action(self.data.get('action'))
+        return Action(self)
 
-    @property
+    @cacheable_resource
     def authentication_options(self):
         """
         Read only authentication options field
 
-        :return: :py:class:`smc.policy.rule_elements.AuthenticationOptions`
+        :rtype: AuthenticationOptions
         """
-        return AuthenticationOptions(self.data.get('authentication_options'))
+        return AuthenticationOptions(self)
 
     @property
     def comment(self):
@@ -134,13 +169,17 @@ class Rule(object):
     def comment(self, value):
         self.data['comment'] = value
 
+    @property
+    def is_rule_section(self):
+        return 'action' not in self.data
+    
     @property
     def is_disabled(self):
         """
         Whether the rule is enabled or disabled
 
         :param bool value: True, False
-        :rtype: boolean
+        :rtype: bool
         """
         return self.data.get('is_disabled')
 
@@ -156,31 +195,49 @@ class Rule(object):
         """
         self.data['is_disabled'] = False
 
-    @property
+    @cacheable_resource
+    def sources(self):
+        """
+        Sources assigned to this rule
+
+        :rtype: Source
+        """
+        return Source(self)
+    
+    @cacheable_resource
     def destinations(self):
         """
         Destinations for this rule
 
-        :return: :py:class:`smc.policy.rule_elements.Destination`
+        :rtype: Destination
         """
-        return Destination(self.data.get('destinations'))
+        return Destination(self)
+    
+    @cacheable_resource
+    def services(self):
+        """
+        Services assigned to this rule
 
-    @property
+        :rtype: Service
+        """
+        return Service(self)
+    
+    @cacheable_resource
     def options(self):
         """
         Rule based options for logging. Enabling and
         disabled specific log settings.
 
-        :return: :py:class:`smc.policy.rule_elements.LogOptions`
+        :rtype: LogOptions
         """
-        return LogOptions(self.data.get('options'))
+        return LogOptions(self)
 
     @property
     def parent_policy(self):
         """
         Read-only name of the parent policy
 
-        :return: :py:class:`smc.base.model.Element` of type policy
+        :return: :class:`smc.base.model.Element` of type policy
         """
         return Element.from_href(self.data.get('parent_policy'))
 
@@ -194,24 +251,10 @@ class Rule(object):
         :return: None
         """
         self.update(PolicyCommandFailed)
-
-    @property
-    def services(self):
-        """
-        Services assigned to this rule
-
-        :return: :py:class:`smc.policy.rule_elements.Service`
-        """
-        return Service(self.data.get('services'))
-
-    @property
-    def sources(self):
-        """
-        Sources assigned to this rule
-
-        :return: :py:class:`smc.policy.rule_elements.Source`
-        """
-        return Source(self.data.get('sources'))
+        try:
+            del self._cache
+        except AttributeError:
+            pass
 
     @property
     def tag(self):
@@ -236,7 +279,130 @@ class Rule(object):
     #        return TimeRange(self.data.get('time_range'))
 
 
-class IPv4Rule(Rule, SubElement):
+class RuleCommon(object):
+    """
+    Functionality common to all rules
+    """
+    def create_rule_section(self, name, add_pos=None, after=None, before=None):
+        """
+        Create a rule section in a Firewall Policy. To specify a specific numbering
+        position for the rule section, use the `add_pos` field. If no position or
+        before/after is specified, the rule section will be placed at the top which
+        will encapsulate all rules below.
+        Create a rule section for the relavant policy::
+        
+            policy = FirewallPolicy('mypolicy')
+            policy.fw_ipv4_access_rules.create_rule_section(name='attop')
+            # For NAT rules
+            policy.fw_ipv4_nat_rules.create_rule_section(name='mysection', add_pos=5)
+        
+        :param str name: create a rule section by name
+        :param int add_pos: position to insert the rule, starting with position 1.
+            If the position value is greater than the number of rules, the rule is
+            inserted at the bottom. If add_pos is not provided, rule is inserted in
+            position 1. Mutually exclusive with ``after`` and ``before`` params. 
+        :param str after: Rule tag to add this rule after. Mutually exclusive with
+            ``add_pos`` and ``before`` params. 
+        :param str before: Rule tag to add this rule before. Mutually exclusive with
+            ``add_pos`` and ``after`` params. 
+        :raises MissingRequiredInput: when options are specified the need additional  
+            setting, i.e. use_vpn action requires a vpn policy be specified. 
+        :raises CreateRuleFailed: rule creation failure 
+        :return: the created ipv4 rule 
+        :rtype: IPv4Rule 
+        """
+        href = self.href
+        params = None
+        if add_pos is not None:
+            href = self.add_at_position(add_pos)
+        elif before or after: 
+            params = self.add_before_after(before, after) 
+    
+        return SubElementCreator( 
+            self.__class__, 
+            CreateRuleFailed, 
+            href=href, 
+            params=params, 
+            json={'comment': name})
+    
+    def add_at_position(self, pos):
+        if pos <= 0:
+            pos = 1
+        rules = self.make_request(href=self.href)
+        if rules:
+            if len(rules) >= pos:  # Position somewhere in the list
+                for position, entry in enumerate(rules):
+                    if position + 1 == pos:
+                        return self.__class__(**entry).get_relation('add_before')
+            else:  # Put at the end
+                last_rule = rules.pop()
+                return self.__class__(**last_rule).get_relation('add_after')
+        return self.href
+
+    def add_before_after(self, before=None, after=None):
+        params = None
+        if after is not None:
+            params = {'after': after}
+        elif before is not None:
+            params = {'before': before}
+        return params
+        
+    def update_targets(self, sources, destinations, services):
+        source = Source()
+        destination = Destination()
+        service = Service()
+    
+        if sources is not None:
+            if isinstance(sources, str) and sources.lower() == 'any':
+                source.set_any()
+            else:
+                source.add_many(sources)
+        else:
+            source.set_none()
+    
+        if destinations is not None:
+            if isinstance(destinations, str) and destinations.lower() == 'any':
+                destination.set_any()
+            else:
+                destination.add_many(destinations)
+        else:
+            destination.set_none()
+    
+        if services is not None:
+            if isinstance(services, str) and services.lower() == 'any':
+                service.set_any()
+            else:
+                service.add_many(services)
+        else:
+            service.set_none()
+    
+        e = {}
+        #e.update(source())
+        e.update(sources=source.data)
+        e.update(destinations=destination.data)
+        e.update(services=service.data)
+        return e
+    
+    def update_logical_if(self, logical_interfaces):
+        e = {}
+        if logical_interfaces is None:
+            e.update(logical_interfaces={'any': True})
+        else:
+            try:
+                logicals = []
+                for interface in logical_interfaces:
+                    logicals.append(LogicalInterface(interface).href)
+                e.update(logical_interfaces={
+                    'logical_interface': logicals})
+            
+            except ElementNotFound:
+                raise MissingRequiredInput(
+                    'Cannot find Logical interface specified '
+                    ': {}'.format(logical_interfaces))
+        return e
+        
+
+class IPv4Rule(RuleCommon, Rule, SubElement):
     """ 
     Represents an IPv4 Rule for a layer 3 engine.
 
@@ -280,7 +446,7 @@ class IPv4Rule(Rule, SubElement):
 
     Services by application (get all facebook applications)::
 
-        >>> applications = Search('application_situation').objects.filter('facebook')
+        >>> applications = Search.objects.entry_point('application_situation').filter('facebook')
         >>> print(list(applications))
         [ApplicationSituation(name=Facebook-Plugins-Share-Button), ApplicationSituation(name=Facebook-Plugins]
         ...
@@ -291,34 +457,36 @@ class IPv4Rule(Rule, SubElement):
         sources='any'
     """
     typeof = 'fw_ipv4_access_rule'
-
-    def __init__(self, **meta):
-        super(IPv4Rule, self).__init__(**meta)
-        self._actions = ['allow', 'discard', 'continue',
-                         'refuse', 'jump', 'apply_vpn',
-                         'enforce_vpn', 'forward_vpn',
-                         'blacklist']
-
+    _actions = ('allow', 'discard', 'continue', 'refuse', 'jump',
+                'apply_vpn', 'enforce_vpn', 'forward_vpn', 'blacklist')
+    
     def create(self, name, sources=None, destinations=None,
                services=None, action='allow', log_options=None,
-               is_disabled=False, vpn_policy=None, add_pos=None,
-               after=None, before=None, sub_policy=None, **kwargs):
+               authentication_options=None, connection_tracking=None,
+               is_disabled=False, vpn_policy=None, mobile_vpn=False,
+               add_pos=None, after=None, before=None,
+               sub_policy=None, comment=None, **kw):
         """ 
         Create a layer 3 firewall rule
 
         :param str name: name of rule
-        :param list[str, Element] sources: source/s for rule
+        :param sources: source/s for rule
         :type sources: list[str, Element]
         :param destinations: destination/s for rule
         :type destinations: list[str, Element]
         :param services: service/s for rule
         :type services: list[str, Element]
         :param action: allow,continue,discard,refuse,enforce_vpn,
-            apply_vpn,blacklist (default: allow)
+            apply_vpn,forward_vpn, blacklist (default: allow)
         :type action: Action or str
         :param LogOptions log_options: LogOptions object
-        :param str: vpn_policy: vpn policy name; required for enforce_vpn and apply_vpn 
-               actions
+        :param ConnectionTracking connection_tracking: custom connection tracking settings
+        :param AuthenticationOptions authentication_options: options for auth if any
+        :param PolicyVPN,str vpn_policy: policy element or str href; required for
+            enforce_vpn, use_vpn and apply_vpn actions
+        :param bool mobile_vpn: if using a vpn action, you can set mobile_vpn to True and
+            omit the vpn_policy setting if you want this VPN to apply to any mobile VPN based
+            on the policy VPN associated with the engine
         :param str,Element sub_policy: sub policy required when rule has an action of 'jump'.
             Can be the FirewallSubPolicy element or href.
         :param int add_pos: position to insert the rule, starting with position 1. If
@@ -329,14 +497,15 @@ class IPv4Rule(Rule, SubElement):
             and ``before`` params.
         :param str before: Rule tag to add this rule before. Mutually exclusive with ``add_pos``
             and ``after`` params.
+        :param str comment: optional comment for this rule
         :raises MissingRequiredInput: when options are specified the need additional 
             setting, i.e. use_vpn action requires a vpn policy be specified.
         :raises CreateRuleFailed: rule creation failure
-        :return: href of new element
-        :rtype: str
+        :return: the created ipv4 rule
+        :rtype: IPv4Rule
         """
-        rule_values = _rule_common(sources, destinations, services)
-        rule_values.update(name=name)
+        rule_values = self.update_targets(sources, destinations, services)
+        rule_values.update(name=name, comment=comment)
 
         if isinstance(action, Action):
             rule_action = action
@@ -346,58 +515,61 @@ class IPv4Rule(Rule, SubElement):
 
         if not rule_action.action in self._actions:
             raise CreateRuleFailed('Action specified is not valid for this '
-                                   'rule type; action: {}'
-                                   .format(rule_action.action))
+                'rule type; action: {}'.format(rule_action.action))
 
-        if rule_action.action in ['apply_vpn', 'enforce_vpn', 'forward_vpn']:
-            if vpn_policy is None:
-                raise MissingRequiredInput('A VPN policy must be specified when '
-                                           'rule action has a VPN action')
-            try:
-                vpn = VPNPolicy(vpn_policy).href
-                rule_action.vpn = vpn
-            except ElementNotFound:
-                raise MissingRequiredInput('Cannot find VPN policy specified: {}, '
-                                           .format(vpn_policy))
-        elif rule_action.action in ['jump']:
+        if rule_action.action in ('apply_vpn', 'enforce_vpn', 'forward_vpn'):
+            if vpn_policy is None and not mobile_vpn:
+                raise MissingRequiredInput('You must either specify a vpn_policy or set '
+                    'mobile_vpn when using a rule with a VPN action')
+            if mobile_vpn:
+                rule_action.mobile_vpn = True
+            else:
+                try:
+                    vpn = element_resolver(vpn_policy) # VPNPolicy
+                    rule_action.vpn = vpn
+                except ElementNotFound:
+                    raise MissingRequiredInput('Cannot find VPN policy specified: {}, '
+                        .format(vpn_policy))
+        
+        elif rule_action.action == 'jump':
             try:
                 rule_action.sub_policy = element_resolver(sub_policy)
             except ElementNotFound:
                 raise MissingRequiredInput('Cannot find sub policy specified: {} '
-                                           .format(sub_policy))
+                    .format(sub_policy))
         
-        rule_values.update(rule_action())
+        #rule_values.update(action=rule_action.data)
 
-        if log_options is None:
-            log_options = LogOptions()
+        log_options = LogOptions() if not log_options else log_options
+        
+        if connection_tracking is not None:
+            rule_action.connection_tracking_options.update(**connection_tracking)
+        
+        auth_options = AuthenticationOptions() if not authentication_options \
+            else authentication_options
 
-        auth_options = AuthenticationOptions()
-
-        rule_values.update(log_options())
-        rule_values.update(auth_options())
-        rule_values.update(is_disabled=is_disabled)
-
-        if add_pos is not None:
-            href = _add_position(add_pos, self.href)
-        else:
-            href = self.href
+        rule_values.update(
+            action=rule_action.data,
+            options=log_options.data,
+            authentication_options=auth_options.data,
+            is_disabled=is_disabled)
 
         params = None
-        if not add_pos:
-            if after is not None:
-                params = {'after': after}
-            elif before is not None:
-                params = {'before': before}
-
-        return prepared_request(
+        href = self.href
+        if add_pos is not None:
+            href = self.add_at_position(add_pos)
+        elif before or after:
+            params = self.add_before_after(before, after)
+    
+        return SubElementCreator(
+            self.__class__,
             CreateRuleFailed,
             href=href,
             params=params,
-            json=rule_values
-        ).create().href
+            json=rule_values)
 
-
-class IPv4Layer2Rule(Rule, SubElement):
+    
+class IPv4Layer2Rule(RuleCommon, Rule, SubElement):
     """
     Create IPv4 rules for Layer 2 Firewalls
 
@@ -410,15 +582,13 @@ class IPv4Layer2Rule(Rule, SubElement):
                                                services='any')
     """
     typeof = 'layer2_ipv4_access_rule'
-
-    def __init__(self, **meta):
-        super(IPv4Layer2Rule, self).__init__(**meta)
-        self._actions = ['allow', 'continue', 'discard',
-                         'refuse', 'jump', 'blacklist']
-
+    _actions = ('allow', 'continue', 'discard',
+                'refuse', 'jump', 'blacklist')
+    
     def create(self, name, sources=None, destinations=None,
                services=None, action='allow', is_disabled=False,
-               logical_interfaces=None, add_pos=None):
+               logical_interfaces=None, add_pos=None,
+               after=None, before=None, comment=None):
         """
         Create an IPv4 Layer 2 FW rule
 
@@ -429,16 +599,25 @@ class IPv4Layer2Rule(Rule, SubElement):
         :type destinations: list[str, Element]
         :param services: service/s for rule
         :type services: list[str, Element]
-        :param list logical_interfaces: logical interfaces by name
         :param str, Action action: \|allow\|continue\|discard\|refuse\|blacklist
         :param bool is_disabled: whether to disable rule or not
+        :param list logical_interfaces: logical interfaces by name
+        :param int add_pos: position to insert the rule, starting with position 1. If
+            the position value is greater than the number of rules, the rule is inserted at
+            the bottom. If add_pos is not provided, rule is inserted in position 1. Mutually
+            exclusive with ``after`` and ``before`` params.
+        :param str after: Rule tag to add this rule after. Mutually exclusive with ``add_pos``
+            and ``before`` params.
+        :param str before: Rule tag to add this rule before. Mutually exclusive with ``add_pos``
+            and ``after`` params.
+        :param str comment: optional comment for this rule
         :raises MissingRequiredInput: when options are specified the need additional
             setting, i.e. use_vpn action requires a vpn policy be specified.
         :raises CreateRuleFailed: rule creation failure
-        :return: href of new element
-        :rtype: str
+        :return: newly created rule
+        :rtype: IPv4Layer2Rule
         """
-        rule_values = _rule_common(sources, destinations, services)
+        rule_values = self.update_targets(sources, destinations, services)
         rule_values.update(name=name)
         rule_values.update(is_disabled=is_disabled)
 
@@ -450,26 +629,29 @@ class IPv4Layer2Rule(Rule, SubElement):
 
         if not rule_action.action in self._actions:
             raise CreateRuleFailed('Action specified is not valid for this '
-                                   'rule type; action: {}'
-                                   .format(rule_action.action))
+                'rule type; action: {}'.format(rule_action.action))
 
-        rule_values.update(rule_action())
+        rule_values.update(action=rule_action.data)
 
-        rule_values.update(_rule_l2_common(logical_interfaces))
+        rule_values.update(self.update_logical_if(logical_interfaces))
 
+        params = None
+        
+        href = self.href
         if add_pos is not None:
-            href = _add_position(add_pos, self.href)
-        else:
-            href = self.href
-
-        return prepared_request(
+            href = self.add_at_position(add_pos)
+        elif before or after:
+            params = self.add_before_after(before, after)
+ 
+        return SubElementCreator(
+            self.__class__,
             CreateRuleFailed,
             href=href,
-            json=rule_values
-        ).create().href
+            params=params,
+            json=rule_values)
 
 
-class EthernetRule(Rule, SubElement):
+class EthernetRule(RuleCommon, Rule, SubElement):
     """
     Ethernet Rule represents a policy on a layer 2 or IPS engine.
 
@@ -485,14 +667,12 @@ class EthernetRule(Rule, SubElement):
                                             action='discard')
     """
     typeof = 'ethernet_rule'
-
-    def __init__(self, **meta):
-        super(EthernetRule, self).__init__(**meta)
-        self._actions = ['allow', 'discard']
+    _actions = ('allow', 'discard')
 
     def create(self, name, sources=None, destinations=None,
                services=None, action='allow', is_disabled=False,
-               logical_interfaces=None, add_pos=None):
+               logical_interfaces=None, add_pos=None,
+               after=None, before=None, comment=None):
         """
         Create an Ethernet rule
 
@@ -503,17 +683,25 @@ class EthernetRule(Rule, SubElement):
         :type destinations: list[str, Element]
         :param services: service/s for rule
         :type services: list[str, Element]
-        :param list logical_interfaces: logical interfaces by name
         :param str action: \|allow\|continue\|discard\|refuse\|blacklist
         :param bool is_disabled: whether to disable rule or not
+        :param list logical_interfaces: logical interfaces by name
+        :param int add_pos: position to insert the rule, starting with position 1. If
+            the position value is greater than the number of rules, the rule is inserted at
+            the bottom. If add_pos is not provided, rule is inserted in position 1. Mutually
+            exclusive with ``after`` and ``before`` params.
+        :param str after: Rule tag to add this rule after. Mutually exclusive with ``add_pos``
+            and ``before`` params.
+        :param str before: Rule tag to add this rule before. Mutually exclusive with ``add_pos``
+            and ``after`` params.
         :raises MissingReuqiredInput: when options are specified the need additional
             setting, i.e. use_vpn action requires a vpn policy be specified.
         :raises CreateRuleFailed: rule creation failure
-        :return: href of new element
-        :rtype: str
+        :return: newly created rule
+        :rtype: EthernetRule
         """
-        rule_values = _rule_common(sources, destinations, services)
-        rule_values.update(name=name)
+        rule_values = self.update_targets(sources, destinations, services)
+        rule_values.update(name=name, comment=comment)
         rule_values.update(is_disabled=is_disabled)
 
         if isinstance(action, Action):
@@ -527,20 +715,24 @@ class EthernetRule(Rule, SubElement):
                                    'rule type; action: {}'
                                    .format(rule_action.action))
 
-        rule_values.update(rule_action())
+        rule_values.update(action=rule_action.data)
 
-        rule_values.update(_rule_l2_common(logical_interfaces))
+        rule_values.update(self.update_logical_if(logical_interfaces))
 
+        params = None
+        
+        href = self.href
         if add_pos is not None:
-            href = _add_position(add_pos, self.href)
+            href = self.add_at_position(add_pos)
         else:
-            href = self.href
+            params = self.add_before_after(before, after)
 
-        return prepared_request(
+        return SubElementCreator(
+            self.__class__,
             CreateRuleFailed,
             href=href,
-            json=rule_values
-        ).create().href
+            params=params,
+            json=rule_values)
 
 
 class IPv6Rule(IPv4Rule):
@@ -553,89 +745,3 @@ class IPv6Rule(IPv4Rule):
               attempting to push policy.
     """
     typeof = 'fw_ipv6_access_rule'
-
-    def __init__(self, **meta):
-        super(IPv6Rule, self).__init__(**meta)
-        pass
-
-
-def _add_position(pos, rules_href):
-    """
-    Add the position for the rule. If the position is a larger number
-    than number of rules, it will be placed at the end. Otherwise
-    inserted into the position specified.
-    """
-    if pos <= 0:
-        pos = 1
-    rules = prepared_request(
-        FetchElementFailed, href=rules_href).read().json
-    if rules:
-        if len(rules) >= pos:  # Position somewhere in the list
-            for position, entry in enumerate(rules):
-                if position + 1 == pos:
-                    return SubElement(**entry).data.get_link('add_before')
-        else:  # Put at the end
-            last_rule = rules.pop()
-            return SubElement(**last_rule).data.get_link('add_after')
-    return rules_href
-
-
-def _rule_l2_common(logical_interfaces):
-    """
-    Common values for layer 2 ethernet / IPS rule parameters.
-    In particular, logical interfaces are an additional parameter that 
-    can be used as a rule match parameter.
-    """
-    rule_values = {}
-    if logical_interfaces is None:
-        rule_values.update(logical_interfaces={'any': True})
-    else:
-        try:
-            logicals = []
-            for interface in logical_interfaces:
-                logicals.append(LogicalInterface(interface).href)
-            rule_values.update(logical_interfaces={
-                               'logical_interface': logicals})
-        except ElementNotFound:
-            raise MissingRequiredInput('Cannot find Logical interface specified '
-                                       ': {}'.format(logical_interfaces))
-    return rule_values
-
-
-def _rule_common(sources, destinations, services):
-    """
-    Common rule elements
-    """
-    source = Source()
-    destination = Destination()
-    service = Service()
-
-    if sources is not None:
-        if isinstance(sources, str) and sources.lower() == 'any':
-            source.set_any()
-        else:
-            source.add_many(sources)
-    else:
-        source.set_none()
-
-    if destinations is not None:
-        if isinstance(destinations, str) and destinations.lower() == 'any':
-            destination.set_any()
-        else:
-            destination.add_many(destinations)
-    else:
-        destination.set_none()
-
-    if services is not None:
-        if isinstance(services, str) and services.lower() == 'any':
-            service.set_any()
-        else:
-            service.add_many(services)
-    else:
-        service.set_none()
-
-    e = {}
-    e.update(source())
-    e.update(destination())
-    e.update(service())
-    return e
