@@ -12,7 +12,7 @@ For adding OSPF configurations, several steps are required:
 
 Enable OSPF on an existing engine using the default OSPF system profile::
 
-    engine.enable_ospf()
+    engine.ospf.enable()
     
 Create an OSPFArea using the default OSPF Interface Setting profile::
 
@@ -25,14 +25,14 @@ Add OSPF area to an interface routing configuration (add to nicid '0')::
                 
 Disable OSPF on an engine::
     
-    engine.disable_ospf()
+    engine.ospf.disable()
 
 Finding profiles or elements can also be done through collections::
 
-    >>> list(Search('ospfv2_profile').objects.all())
+    >>> list(OSPFProfile.objects.all())
     [OSPFProfile(name=Default OSPFv2 Profile)]
         
-    >>> list(Search('ospfv2_area').objects.all())
+    >>> list(OSPFArea.objects.all())
     [OSPFArea(name=area0)]
 
 The OSPF relationship can be represented as::
@@ -46,10 +46,102 @@ Only Layer3Firewall and Layer3VirtualEngine types can support running OSPF.
              :py:class:`smc.core.engines.Layer3VirtualEngine`
 
 """
-from smc.base.model import Element, ElementCreator
+from smc.base.model import Element, ElementCreator, ElementCache, ElementRef
 from smc.base.util import element_resolver
 
 
+class OSPF(object):
+    """
+    OSPF configuration on the engine. Access through an engine reference::
+        
+        engine.dynamic_routing.ospf.status
+        engine.dynamic_rotuing.ospf.enable(....)
+        
+    When making changes to the OSPF configuration, any methods
+    called that change the configuration also require that
+    engine.update() is called once changes are complete. This way
+    you can make multiple changes without refreshing the engine cache.
+    
+    :ivar OSPFProfile profile: OSPFProfile reference for this engine
+    """
+    profile = ElementRef('ospfv2_profile_ref')
+    
+    def __init__(self, data=None):
+        self.data = data if data else ElementCache()
+    
+    @property
+    def router_id(self):
+        """
+        Get the router ID for this OSPF configuration. If None, then
+        the ID will use the interface IP.
+        
+        :return: str or None
+        """
+        return self.data.get('router_id')
+    
+    @property
+    def status(self):
+        """
+        Is OSPF enabled on this engine.
+        
+        :rtype: bool
+        """
+        return self.data.get('enabled')
+    
+    def disable(self):
+        """
+        Disable OSPF on this engine.
+
+        :return: None
+        """
+        self.data.update(
+            enabled=False)
+
+    def enable(self, ospf_profile=None, router_id=None):
+        """
+        Enable OSPF on this engine. For master engines, enable
+        OSPF on the virtual firewall.
+
+        Once enabled on the engine, add an OSPF area to an interface::
+
+            engine.dynamic_routing.ospf.enable()
+            interface = engine.routing.get(0)
+            interface.add_ospf_area(OSPFArea('myarea'))
+
+        :param str,OSPFProfile ospf_profile: OSPFProfile element or str
+            href; if None, use default profile
+        :param str router_id: single IP address router ID
+        :raises ElementNotFound: OSPF profile not found
+        :return: None
+        """
+        ospf_profile = element_resolver(ospf_profile) if ospf_profile \
+            else OSPFProfile('Default OSPFv2 Profile').href
+
+        self.data.update(
+            enabled=True,
+            ospfv2_profile_ref=ospf_profile,
+            router_id=router_id)
+    
+    def update_configuration(self, **kwargs):
+        """
+        Update the OSPF configuration using kwargs that match the
+        `enable` constructor.
+        
+        :param dict kwargs: keyword arguments matching enable constructor.
+        :return: whether change was made
+        :rtype: bool
+        """
+        updated = False
+        if 'ospf_profile' in kwargs:
+            kwargs.update(ospfv2_profile_ref=kwargs.pop('ospf_profile'))
+        for name, value in kwargs.items():
+            _value = element_resolver(value)
+            if self.data.get(name) != _value:
+                self.data[name] = _value
+                updated = True
+        return updated
+
+
 class OSPFArea(Element):
     """
     OSPF Area is an element that identifies general settings for an
@@ -101,20 +193,19 @@ class OSPFArea(Element):
                      'substitute_type': 'substitute_with'},
                     {'subnet_ref': 'http://172.18.1.150:8082/6.1/elements/network/979',
                      'substitute_type': 'aggregate'}])
+    
+    :ivar OSPFInterfaceSetting interface_settings_ref: reference to the OSPFInterfaceSetting
+        element
     """
     typeof = 'ospfv2_area'
-
-    def __init__(self, name, **meta):
-        super(OSPFArea, self).__init__(name, **meta)
-        pass
+    interface_settings_ref = ElementRef('interface_settings_ref')
 
     @classmethod
     def create(cls, name, interface_settings_ref=None, area_id=1,
-               area_default_cost=100, area_type='normal',
-               outbound_filters_ref=None, inbound_filters_ref=None,
-               shortcut_capable_area=False,
+               area_type='normal', outbound_filters=None,
+               inbound_filters=None, shortcut_capable_area=False,
                ospfv2_virtual_links_endpoints_container=None,
-               ospf_abr_substitute_container=None):
+               ospf_abr_substitute_container=None, comment=None, **kwargs):
         """
         Create a new OSPF Area
 
@@ -122,43 +213,74 @@ class OSPFArea(Element):
         :param str,OSPFInterfaceSetting interface_settings_ref: an OSPFInterfaceSetting
             element or href. If None, uses the default system profile
         :param str name: area id
-        :param int area_default_cost: default cost for this area
         :param str area_type: \|normal\|stub\|not_so_stubby\|totally_stubby\|
                totally_not_so_stubby
-        :param str outbound_filters_ref: reference to 
-               :py:class:`~smc.routing.access_list.IPAccessList`
-        :param str inbound_filters_ref: reference to
-               :py:class:`~smc.routing.access_list.IPAccessList`
+        :param list outbound_filters: reference to an IPAccessList and or IPPrefixList.
+            You can only have one outbound prefix or access list
+        :param list inbound_filters: reference to an IPAccessList and or IPPrefixList.
+            You can only have one outbound prefix or access list
         :param shortcut_capable_area: True|False
         :param list ospfv2_virtual_links_endpoints_container: virtual link endpoints
         :param list ospf_abr_substitute_container: substitute types: 
                \|aggregate\|not_advertise\|substitute_with
+        :param str comment: optional comment
         :raises CreateElementFailed: failed to create with reason
-        :return: instance with meta
         :rtype: OSPFArea
         """
-        if interface_settings_ref is None:
-            interface_settings_ref = \
-                OSPFInterfaceSetting('Default OSPFv2 Interface Settings').href
-        else:
-            interface_settings_ref = element_resolver(interface_settings_ref)
-
+        interface_settings_ref = element_resolver(interface_settings_ref) or \
+            OSPFInterfaceSetting('Default OSPFv2 Interface Settings').href
+        
+        if 'inbound_filters_ref' in kwargs:
+            inbound_filters = kwargs.get('inbound_filters_ref')
+        
+        if 'outbound_filters_ref' in kwargs:
+            outbound_filters = kwargs.get('outbound_filters_ref')
+        
         json = {'name': name,
                 'area_id': area_id,
                 'area_type': area_type,
-                'inbound_filters_ref': inbound_filters_ref,
+                'comment': comment,
+                'inbound_filters_ref': element_resolver(inbound_filters),
                 'interface_settings_ref': interface_settings_ref,
                 'ospf_abr_substitute_container': ospf_abr_substitute_container,
                 'ospfv2_virtual_links_endpoints_container':
                     ospfv2_virtual_links_endpoints_container,
-                'outbound_filters_ref': outbound_filters_ref,
+                'outbound_filters_ref': element_resolver(outbound_filters),
                 'shortcut_capable_area': shortcut_capable_area}
 
         return ElementCreator(cls, json)
 
+    @classmethod
+    def update_or_create(cls, filter_key=None, with_status=False, **kwargs):
+        if 'inbound_filters' in kwargs:
+            kwargs.update(inbound_filters_ref=
+                element_resolver(kwargs.pop('inbound_filters')))
+        if 'outbound_filters' in kwargs:
+            kwargs.update(outbound_filters_ref=
+                element_resolver(kwargs.pop('outbound_filters')))
+        return super(OSPFArea, cls).update_or_create(filter_key, with_status, **kwargs)
+
     @property
-    def interface_settings_ref(self):
-        return Element.from_href(self.data.get('interface_settings_ref'))
+    def inbound_filters(self):
+        """
+        Inbound filters attached to this OSPF Area. Filters can be type
+        IPPrefixList or IPAccessList
+        
+        :rtype: list
+        """
+        return [Element.from_href(filt)
+            for filt in self.data.get('inbound_filters_ref', [])]
+    
+    @property
+    def outbound_filters(self):
+        """
+        Outbound filters attached to this OSPF Area. Filters can be type
+        IPPrefixList or IPAccessList
+        
+        :rtype: list
+        """
+        return [Element.from_href(filt)
+            for filt in self.data.get('outbound_filters_ref', [])]
 
 
 class OSPFInterfaceSetting(Element):
@@ -185,10 +307,7 @@ class OSPFInterfaceSetting(Element):
     parameter must be specified.
     """
     typeof = 'ospfv2_interface_settings'
-
-    def __init__(self, name, **meta):
-        super(OSPFInterfaceSetting, self).__init__(name, **meta)
-        pass
+    key_chain_ref = ElementRef('key_chain_ref')
 
     @classmethod
     def create(cls, name, dead_interval=40, hello_interval=10,
@@ -218,11 +337,10 @@ class OSPFInterfaceSetting(Element):
         :return: instance with meta
         :rtype: OSPFInterfaceSetting
         """
-        key_chain_ref = element_resolver(key_chain_ref) if key_chain_ref else None
         json = {'name': name,
                 'authentication_type': authentication_type,
                 'password': password,
-                'key_chain_ref': key_chain_ref,
+                'key_chain_ref': element_resolver(key_chain_ref),
                 'dead_interval': dead_interval,
                 'dead_multiplier': dead_multiplier,
                 'hello_interval': hello_interval,
@@ -253,10 +371,6 @@ class OSPFKeyChain(Element):
     """
     typeof = 'ospfv2_key_chain'
 
-    def __init__(self, name, **meta):
-        super(OSPFKeyChain, self).__init__(name, **meta)
-        pass
-
     @classmethod
     def create(cls, name, key_chain_entry):
         """
@@ -272,13 +386,30 @@ class OSPFKeyChain(Element):
         :return: instance with meta
         :rtype: OSPFKeyChain
         """
-        key_chain_entry = [] if key_chain_entry is None else key_chain_entry
+        key_chain_entry = key_chain_entry or []
         json = {'name': name,
                 'ospfv2_key_chain_entry': key_chain_entry}
 
         return ElementCreator(cls, json)
 
 
+def _format_redist_entry(redistribution_entry):
+    for entry in redistribution_entry:
+        _filter = entry.pop('filter', None)
+        if _filter and _filter.typeof == 'ip_access_list':
+            entry.update(
+                filter_type='access_list',
+                redistribution_filter_ref=_filter.href)
+        elif _filter and _filter.typeof == 'route_map':
+            entry.update(
+                filter_type='route_map_policy',
+                redistribution_rm_ref=_filter.href)
+        if not 'metric_type' in entry:
+            entry.update(
+                metric_type='external_1')
+    return redistribution_entry
+
+    
 class OSPFProfile(Element):
     """
     An OSPF Profile contains administrative distance and redistribution 
@@ -293,21 +424,46 @@ class OSPFProfile(Element):
 
         OSPFProfile.create(name='myospf')
 
-    See :py:class:`smc.core.properties.EngineFeature.enable_ospf` for enabling 
-    ospf on an existing engine.
-
+    .. note:: Enable OSPF on engine using engine.ospf.enable()
+    
+    :ivar int external_distance: external distance metric
+    :ivar int inter_distance: inter distance metric
+    :ivar int intra_distance: intra distance metric
+    :ivar int default_metric: set a default metric for all unset areas
+    :ivar list redistribution_entry: settings for static, connected, etc
+    :ivar OSPFDomainSetting domain_settings_ref: OSPF Domain Settings profile
+        used for this OSPF Profile
     """
     typeof = 'ospfv2_profile'
-
-    def __init__(self, name, **meta):
-        super(OSPFProfile, self).__init__(name, **meta)
-        pass
+    domain_settings_ref = ElementRef('domain_settings_ref')
 
     @classmethod
     def create(cls, name, domain_settings_ref=None, external_distance=110,
-               inter_distance=110, intra_distance=110):
+               inter_distance=110, intra_distance=110, redistribution_entry=None,
+               default_metric=None, comment=None):
         """
-        Create an OSPF Profile
+        Create an OSPF Profile.
+        
+        If providing a list of redistribution entries, provide in the following
+        dict format: 
+        
+        {'enabled': boolean, 'metric_type': 'external_1' or 'external_2',
+         'metric': 2, 'type': 'kernel'}
+        
+        Valid types for redistribution entries are: kernel, static, connected, bgp,
+        and default_originate.
+        
+        You can also provide a 'filter' key with either an IPAccessList or RouteMap
+        element to use for further access control on the redistributed route type.
+        If metric_type is not provided, external_1 (E1) will be used. 
+        
+        An example of a redistribution_entry would be::
+        
+            {u'enabled': True,
+             u'metric': 123,
+             u'metric_type': u'external_2',
+             u'filter': RouteMap('myroutemap'),
+             u'type': u'static'}
 
         :param str name: name of profile
         :param str,OSPFDomainSetting domain_settings_ref: OSPFDomainSetting 
@@ -315,60 +471,34 @@ class OSPFProfile(Element):
         :param int external_distance: route metric (E1-E2)
         :param int inter_distance: routes learned from different areas (O IA)
         :param int intra_distance: routes learned from same area (O)
+        :param list redistribution_entry: how to redistribute the OSPF routes.
         :raises CreateElementFailed: create failed with reason
-        :return: instance with meta
         :rtype: OSPFProfile
         """
         json = {'name': name,
                 'external_distance': external_distance,
                 'inter_distance': inter_distance,
-                'intra_distance': intra_distance}
+                'intra_distance': intra_distance,
+                'default_metric': default_metric,
+                'comment': comment}
+        
+        if redistribution_entry:
+            json.update(redistribution_entry=
+                _format_redist_entry(redistribution_entry))
 
-        if not domain_settings_ref:
-            domain_settings_ref = OSPFDomainSetting(
-                'Default OSPFv2 Domain Settings').href
-        else:
-            domain_settings_ref = element_resolver(domain_settings_ref)
+        domain_settings_ref = element_resolver(domain_settings_ref) or \
+            OSPFDomainSetting('Default OSPFv2 Domain Settings').href
 
         json.update(domain_settings_ref=domain_settings_ref)
 
         return ElementCreator(cls, json)
-
-    @property
-    def external_distance(self):
-        """
-        External administrative distance. Between 1-255.
-
-        :return: int distance value
-        """
-        return self.data.get('external_distance')
-
-    @property
-    def inter_distance(self):
-        """
-        Inter administrative distance. Between 1-255.
-
-        :return: int distance value
-        """
-        return self.data.get('inter_distance')
-
-    @property
-    def intra_distance(self):
-        """
-        Intra administrative distance. Between 1-255.
-
-        :return: int distance value
-        """
-        return self.data.get('intra_distance')
-
-    @property
-    def domain_settings_ref(self):
-        """
-        OSPF Domain Settings profile used for this OSPF Profile
-
-        :return: :class:`~OSPFDomainSetting`
-        """
-        return Element.from_href(self.data.get('domain_settings_ref'))
+        
+    @classmethod
+    def update_or_create(cls, filter_key=None, with_status=False, **kwargs):
+        if 'redistribution_entry' in kwargs:
+            kwargs.update(redistribution_entry=_format_redist_entry(
+                kwargs.pop('redistribution_entry', [])))
+        return super(OSPFProfile, cls).update_or_create(filter_key, with_status, **kwargs)
 
 
 class OSPFDomainSetting(Element):
@@ -389,10 +519,6 @@ class OSPFDomainSetting(Element):
     """
     typeof = 'ospfv2_domain_settings'
 
-    def __init__(self, name, **meta):
-        super(OSPFDomainSetting, self).__init__(name, **meta)
-        pass
-
     @classmethod
     def create(cls, name, abr_type='cisco', auto_cost_bandwidth=100,
                deprecated_algorithm=False, initial_delay=200,
